# This file is automatically generated. Please do not modify.

from . import AsciiArt

aix = AsciiArt(match=r'''"AIX"*''', color='2 7', ascii=r"""
${c1}           `:+ssssossossss+-`
        .oys///oyhddddhyo///sy+.
      /yo:+hNNNNNNNNNNNNNNNNh+:oy/
    :h/:yNNNNNNNNNNNNNNNNNNNNNNy-+h:
  `ys.yNNNNNNNNNNNNNNNNNNNNNNNNNNy.ys
 `h+-mNNNNNNNNNNNNNNNNNNNNNNNNNNNNm-oh
 h+-NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN.oy
/d`mNNNNNNN/::mNNNd::m+:/dNNNo::dNNNd`m:
h//NNNNNNN: . .NNNh  mNo  od. -dNNNNN:+y
N.sNNNNNN+ -N/ -NNh  mNNd.   sNNNNNNNo-m
N.sNNNNNs  +oo  /Nh  mNNs` ` /mNNNNNNo-m
h//NNNNh  ossss` +h  md- .hm/ `sNNNNN:+y
:d`mNNN+/yNNNNNd//y//h//oNNNNy//sNNNd`m-
 yo-NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNm.ss
 `h+-mNNNNNNNNNNNNNNNNNNNNNNNNNNNNm-oy
   sy.yNNNNNNNNNNNNNNNNNNNNNNNNNNs.yo
    :h+-yNNNNNNNNNNNNNNNNNNNNNNs-oh-
      :ys:/yNNNNNNNNNNNNNNNmy/:sy:
        .+ys///osyhhhhys+///sy+.
            -/osssossossso/-
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

solus = AsciiArt(match=r'''"Solus"*''', color='4 7 1', ascii=r"""
${c2}            -```````````
          `-+/------------.`
       .---:mNo---------------.
     .-----yMMMy:---------------.
   `------oMMMMMm/----------------`
  .------/MMMMMMMN+----------------.
 .------/NMMMMMMMMm-+/--------------.
`------/NMMMMMMMMMN-:mh/-------------`
.-----/NMMMMMMMMMMM:-+MMd//oso/:-----.
-----/NMMMMMMMMMMMM+--mMMMh::smMmyo:--
----+NMMMMMMMMMMMMMo--yMMMMNo-:yMMMMd/.
.--oMMMMMMMMMMMMMMMy--yMMMMMMh:-yMMMy-`
`-sMMMMMMMMMMMMMMMMh--dMMMMMMMd:/Ny+y.
`-/+osyhhdmmNNMMMMMm-/MMMMMMMmh+/ohm+
  .------------:://+-/++++++${c1}oshddys:
   -hhhhyyyyyyyyyyyhhhhddddhysssso-
    `:ossssssyysssssssssssssssso:`
      `:+ssssssssssssssssssss+-
         `-/+ssssssssssso+/-`
              `.-----..`
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

januslinux = AsciiArt(match=r'''"januslinux"*|"janus"*|"Ataraxia Linux"*|"Ataraxia"*''', color='4 5 6 2', ascii=r"""
${c1}               'l:
        loooooo
          loooo coooool
 looooooooooooooooooool
  looooooooooooooooo
         lool   cooo
        coooooooloooooooo
     clooooo  ;lood  cloooo
  :loooocooo cloo      loooo
 loooo  :ooooool       loooo
looo    cooooo        cooooo
looooooooooooo      ;loooooo ${c2}looooooc
${c1}looooooooo loo   cloooooool    ${c2}looooc
${c1} cooo       cooooooooooo       ${c2}looolooooool
${c1}            cooo:     ${c2}coooooooooooooooooool
                       loooooooooooolc:   loooc;
                             cooo:    loooooooooooc
                            ;oool         looooooo:
                           coool          olc,
                          looooc   ,,
                        coooooc    loc
                       :oooool,    coool:, looool:,
                       looool:      ooooooooooooooo:
                       cooolc        .ooooooooooool
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

porteus = AsciiArt(match=r'''"Porteus"*''', color='6 7', ascii=r"""
${c1}             `.-:::-.`
         -+ydmNNNNNNNmdy+-
      .+dNmdhs+//////+shdmdo.
    .smmy+-`             ./sdy:
  `omdo.    `.-/+osssso+/-` `+dy.
 `yms.   `:shmNmdhsoo++osyyo-``oh.
 hm/   .odNmds/.`    ``.....:::-+s
/m:  `+dNmy:`   `./oyhhhhyyooo++so
ys  `yNmy-    .+hmmho:-.`     ```
s:  yNm+`   .smNd+.
`` /Nm:    +dNd+`
   yN+   `smNy.
   dm    oNNy`
   hy   -mNm.
   +y   oNNo
   `y`  sNN:
    `:  +NN:
     `  .mNo
         /mm`
          /my`
           .sy`
             .+:
                `
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

maui = AsciiArt(match=r'''"Maui"*''', color='6 7', ascii=r"""
${c1}             `.-://////:--`
         .:/oooooooooooooooo+:.
      `:+ooooooooooooooooooooooo:`
    `:oooooooooooooooooooooooooooo/`
    ..```-oooooo/-`` `:oooooo+:.` `--
  :.      +oo+-`       /ooo/`       -/
 -o.     `o+-          +o/`         -o:
`oo`     ::`  :o/     `+.  .+o`     /oo.
/o+      .  -+oo-     `   /oo/     `ooo/
+o-        /ooo+`       .+ooo.     :ooo+
++       .+oooo:       -oooo+     `oooo+
:.      .oooooo`      :ooooo-     :oooo:
`      .oooooo:      :ooooo+     `ooo+-`
      .+oooooo`     -oooooo:     `o/-
      +oooooo:     .ooooooo.
     /ooooooo`     /ooooooo/       ..
    `:oooooooo/:::/ooooooooo+:--:/:`
      `:+oooooooooooooooooooooo+:`
         .:+oooooooooooooooo+:.
             `.-://////:-.`
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

femboyos = AsciiArt(match=r'''"FemboyOS"*''', color='4', ascii=r"""
${c1}MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMWKkxkKWMMMMMMMMMMMMMMMMMMMMWKkxkKWMM
MMMMXo. .;xKWMMMMMMMMMMMMMMMMMMXo. .oXMM
MMWXx,..'..oXMMMMMMMMMMMMMMMMWKx,  .lXMM
MMNo. .cOc.,xKWMMMMMMMMMMMMWXx;.....cXMM
MMXl..;kKl. .oXMMMMMMMMMMWKx;..,ok:.'o0W
WKx,.cKWNk;..lXMMMMMMMMWKx;..,o0NXl. .oN
No. .lXMMWKc.,dKWMMMMMMNo..;d0NWMNx,..lX
Nk:,:kNMMMNk:,ckNMMMMMMNxcxXWMMMMMN0ockN
MWNNNWMMMMMWNNNWMMMMMMMMWWWMMMMMMMMMWWWM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMNXKXNWMMMMMMMMMMMWNKOKWMMMMMMMMMM
MMMMMMWKdccxXMMMMMMMMMMW0o'.oXMMMMMMMMMM
MMMMMMMNO:.'o0NKkkkkkOXXo. .lXMMMMMMMMMM
MMMMMMMMNx,..;o;.    .:o,..;kNMMMMMMMMMM
MMMMMMMMMNO:     ...     .cKWMMMMMMMMMMM
MMMMMMMMMMNx,. .;dk:.   .;kNMMMMMMMMMMMM
MMMMMMMMMMMN0ocxXWNkl:,:xXWMMMMMMMMMMMMM
MMMMMMMMMMMMMWNWMMMWWNNNWMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
""")
    


from hyfetch.color_scale import test_color_scale
from hyfetch.color_util import RGB, printc
from hyfetch.neofetch_util import get_command_path
from hyfetch.presets import PRESETS


def print_colors_test(colors: list[RGB]):
    print(''.join(f'{c.to_ansi_rgb()}#' for c in colors))


def test_preset_length():
    p = PRESETS.get('transgender')
    print_colors_test(p.with_length(9))
    print_colors_test(p.with_length(6))
    p = PRESETS.get('nonbinary')
    print_colors_test(p.with_length(7))
    print_colors_test(p.with_length(6))


def test_command_path():
    print(get_command_path())


def test_rgb_8bit_conversion():
    for r in range(0, 255, 16):
        for g in range(0, 255, 16):
            print(RGB(r, g, 0).to_ansi_rgb(False), end=' ')
        printc('&*')
    print()
    for r in range(0, 255, 16):
        for g in range(0, 255, 16):
            print(RGB(r, g, 0).to_ansi_8bit(False), end=' ')
        printc('&*')
    print()


if __name__ == '__main__':
    test_rgb_8bit_conversion()
    test_color_scale()



from __future__ import annotations

from typing import Iterable

from .color_util import RGB
from .constants import GLOBAL_CFG
from .types import LightDark, ColorSpacing


def remove_duplicates(seq: Iterable) -> list:
    """
    Remove duplicate items from a sequence while preserving the order
    """
    seen = set()
    seen_add = seen.add
    return [x for x in seq if not (x in seen or seen_add(x))]


class ColorProfile:
    raw: list[str]
    colors: list[RGB]
    spacing: ColorSpacing = 'equal'

    def __init__(self, colors: list[str] | list[RGB]):
        if isinstance(colors[0], str):
            self.raw = colors
            self.colors = [RGB.from_hex(c) for c in colors]
        else:
            self.colors = colors

    def with_weights(self, weights: list[int]) -> list[RGB]:
        """
        Map colors based on weights

        :param weights: Weights of each color (weights[i] = how many times color[i] appears)
        :return:
        """
        return [c for i, w in enumerate(weights) for c in [self.colors[i]] * w]

    def with_length(self, length: int) -> list[RGB]:
        """
        Spread to a specific length of text

        :param length: Length of text
        :return: List of RGBs of the length
        """
        preset_len = len(self.colors)
        center_i = preset_len // 2

        # How many copies of each color should be displayed at least?
        repeats = length // preset_len
        weights = [repeats] * preset_len

        # How many extra space left?
        extras = length % preset_len

        # If there is an even space left, extend the center by one space
        if extras % 2 == 1:
            extras -= 1
            weights[center_i] += 1

        # Add weight to border until there's no space left (extras must be even at this point)
        border_i = 0
        while extras > 0:
            extras -= 2
            weights[border_i] += 1
            weights[-(border_i + 1)] += 1
            border_i += 1

        return self.with_weights(weights)

    def color_text(self, txt: str, foreground: bool = True, space_only: bool = False) -> str:
        """
        Color a text

        :param txt: Text
        :param foreground: Whether the foreground text show the color or the background block
        :param space_only: Whether to only color spaces
        :return: Colored text
        """
        colors = self.with_length(len(txt))
        result = ''
        for i, t in enumerate(txt):
            if space_only and t != ' ':
                if i > 0 and txt[i - 1] == ' ':
                    result += '\033[39;49m'
                result += t
            else:
                result += colors[i].to_ansi(foreground=foreground) + t

        result += '\033[39;49m'
        return result

    def lighten(self, multiplier: float) -> ColorProfile:
        """
        Lighten the color profile by a multiplier

        :param multiplier: Multiplier
        :return: Lightened color profile (original isn't modified)
        """
        return ColorProfile([c.lighten(multiplier) for c in self.colors])

    def set_light_raw(self, light: float, at_least: bool | None = None, at_most: bool | None = None) -> 'ColorProfile':
        """
        Set HSL lightness value

        :param light: Lightness value (0-1)
        :param at_least: Set the lightness to at least this value (no change if greater)
        :param at_most: Set the lightness to at most this value (no change if lesser)
        :return: New color profile (original isn't modified)
        """
        return ColorProfile([c.set_light(light, at_least, at_most) for c in self.colors])

    def set_light_dl(self, light: float, term: LightDark | None = None):
        """
        Set HSL lightness value with respect to dark/light terminals

        :param light: Lightness value (0-1)
        :param term: Terminal color (can be "dark" or "light")
        :return: New color profile (original isn't modified)
        """
        if GLOBAL_CFG.use_overlay:
            return self.overlay_dl(light, term)

        term = term or GLOBAL_CFG.light_dark()
        assert term.lower() in ['light', 'dark']
        at_least, at_most = (True, None) if term.lower() == 'dark' else (None, True)
        return self.set_light_raw(light, at_least, at_most)

    def overlay_raw(self, color: RGB, alpha: float) -> 'ColorProfile':
        """
        Overlay a color on top of the color profile

        :param color: Color to overlay
        :param alpha: Alpha value (0-1)
        :return: New color profile (original isn't modified)
        """
        return ColorProfile([c.overlay(color, alpha) for c in self.colors])

    def overlay_dl(self, light: float, term: LightDark | None = None):
        """
        Same as set_light_dl except that this function uses RGB overlaying instead of HSL lightness change
        """
        term = term or GLOBAL_CFG.light_dark()
        assert term.lower() in ['light', 'dark']

        # If it's light bg, overlay black, else overlay white
        overlay_color = RGB.from_hex('#000000' if term.lower() == 'light' else '#FFFFFF')
        return self.overlay_raw(overlay_color, abs(light - 0.5) * 2)

    def set_light_dl_def(self, term: LightDark | None = None):
        """
        Set default lightness with respect to dark/light terminals

        :param term: Terminal color (can be "dark" or "light")
        :return: New color profile (original isn't modified)
        """
        return self.set_light_dl(GLOBAL_CFG.default_lightness(term), term)

    def unique_colors(self) -> ColorProfile:
        """
        Create another color profile with only the unique colors
        """
        return ColorProfile(remove_duplicates(self.colors))


PRESETS: dict[str, ColorProfile] = {
    'rainbow': ColorProfile([
        '#E50000',
        '#FF8D00',
        '#FFEE00',
        '#028121',
        '#004CFF',
        '#770088'
    ]),

    'transgender': ColorProfile([
        '#55CDFD',
        '#F6AAB7',
        '#FFFFFF',
        '#F6AAB7',
        '#55CDFD'
    ]),

    'nonbinary': ColorProfile([
        '#FCF431',
        '#FCFCFC',
        '#9D59D2',
        '#282828'
    ]),

    # xenogender sourced from https://commons.wikimedia.org/wiki/File:Xenogender_pride_flag.svg
    'xenogender': ColorProfile([
        '#FF6692',
        '#FF9A98',
        '#FFB883',
        '#FBFFA8',
        '#85BCFF',
        '#9D85FF',
        '#A510FF'
    ]),

    'agender': ColorProfile([
        '#000000',
        '#BABABA',
        '#FFFFFF',
        '#BAF484',
        '#FFFFFF',
        '#BABABA',
        '#000000'
    ]),

    'queer': ColorProfile([
        '#B57FDD',
        '#FFFFFF',
        '#49821E'
    ]),

    'genderfluid': ColorProfile([
        '#FE76A2',
        '#FFFFFF',
        '#BF12D7',
        '#000000',
        '#303CBE'
    ]),

    'bisexual': ColorProfile([
        '#D60270',
        '#9B4F96',
        '#0038A8'
    ]),

    'pansexual': ColorProfile([
        '#FF1C8D',
        '#FFD700',
        '#1AB3FF'
    ]),

    'polysexual': ColorProfile([
        '#F714BA',
        '#01D66A',
        '#1594F6',
    ]),

    # omnisexual sorced from https://www.flagcolorcodes.com/omnisexual
    'omnisexual': ColorProfile([
        '#FE9ACE',
        '#FF53BF',
        '#200044',
        '#6760FE',
        '#8EA6FF',
    ]),

    'omniromantic': ColorProfile([
        '#FEC8E4',
        '#FDA1DB',
        '#89739A',
        '#ABA7FE',
        '#BFCEFF',
    ]),

    # gay men sourced from https://www.flagcolorcodes.com/gay-men
    'gay-men': ColorProfile([
        '#078D70',
        '#98E8C1',
        '#FFFFFF',
        '#7BADE2',
        '#3D1A78'
    ]),

    'lesbian': ColorProfile([
        '#D62800',
        '#FF9B56',
        '#FFFFFF',
        '#D462A6',
        '#A40062'
    ]),

    # abrosexual used colorpicker to source from
    # https://fyeahaltpride.tumblr.com/post/151704251345/could-you-guys-possibly-make-an-abrosexual-pride
    'abrosexual': ColorProfile([
        '#46D294',
        '#A3E9CA',
        '#FFFFFF',
        '#F78BB3',
        '#EE1766',
    ]),

    'asexual': ColorProfile([
        '#000000',
        '#A4A4A4',
        '#FFFFFF',
        '#810081'
    ]),

    'aromantic': ColorProfile([
        '#3BA740',
        '#A8D47A',
        '#FFFFFF',
        '#ABABAB',
        '#000000'
    ]),
    
    # https://orientation.fandom.com/wiki/Fictosexual
    'fictosexual': ColorProfile(["#000000", "#C4C4C4", "#A349A5", "#C4C4C4", "#000000"]),

    # aroace1 sourced from https://flag.library.lgbt/flags/aroace/
    'aroace1': ColorProfile([
        '#E28C00',
        '#ECCD00',
        '#FFFFFF',
        '#62AEDC',
        '#203856'
    ]),

    'aroace2': ColorProfile([
        '#000000',
        '#810081',
        '#A4A4A4',
        '#FFFFFF',
        '#A8D47A',
        '#3BA740'
    ]),

    'aroace3': ColorProfile([
        '#3BA740',
        '#A8D47A',
        '#FFFFFF',
        '#ABABAB',
        '#000000',
        '#A4A4A4',
        '#FFFFFF',
        '#810081'
    ]),

    # below sourced from https://www.flagcolorcodes.com/flags/pride
    # goto f"https://www.flagcolorcodes.com/{preset}" for info
    # todo: sane sorting
    'autosexual': ColorProfile([
        '#99D9EA',
        '#7F7F7F'
    ]),

    'intergender': ColorProfile([
        # todo: use weighted spacing
        '#900DC2',
        '#900DC2',
        '#FFE54F',
        '#900DC2',
        '#900DC2',
    ]),

    'greygender': ColorProfile([
        '#B3B3B3',
        '#B3B3B3',
        '#FFFFFF',
        '#062383',
        '#062383',
        '#FFFFFF',
        '#535353',
        '#535353',
    ]),

    'akiosexual': ColorProfile([
        '#F9485E',
        '#FEA06A',
        '#FEF44C',
        '#FFFFFF',
        '#000000',
    ]),

    # bigender sourced from https://www.flagcolorcodes.com/bigender
    'bigender': ColorProfile([
        '#C479A2',
        '#EDA5CD',
        '#D6C7E8',
        '#FFFFFF',
        '#D6C7E8',
        '#9AC7E8',
        '#6D82D1',
    ]),

    # demigender yellow sourced from https://lgbtqia.fandom.com/f/p/4400000000000041031
    # other colors sourced from demiboy and demigirl flags
    'demigender': ColorProfile([
        '#7F7F7F',
        '#C4C4C4',
        '#FBFF75',
        '#FFFFFF',
        '#FBFF75',
        '#C4C4C4',
        '#7F7F7F',
    ]),

    # demiboy sourced from https://www.flagcolorcodes.com/demiboy
    'demiboy': ColorProfile([
        '#7F7F7F',
        '#C4C4C4',
        '#9DD7EA',
        '#FFFFFF',
        '#9DD7EA',
        '#C4C4C4',
        '#7F7F7F',
    ]),

    # demigirl sourced from https://www.flagcolorcodes.com/demigirl
    'demigirl': ColorProfile([
        '#7F7F7F',
        '#C4C4C4',
        '#FDADC8',
        '#FFFFFF',
        '#FDADC8',
        '#C4C4C4',
        '#7F7F7F',
    ]),

    'transmasculine': ColorProfile([
        '#FF8ABD',
        '#CDF5FE',
        '#9AEBFF',
        '#74DFFF',
        '#9AEBFF',
        '#CDF5FE',
        '#FF8ABD',
    ]),

    # transfeminine used colorpicker to source from https://www.deviantart.com/pride-flags/art/Trans-Woman-Transfeminine-1-543925985
    # linked from https://gender.fandom.com/wiki/Transfeminine
    'transfeminine': ColorProfile([
        '#73DEFF',
        '#FFE2EE',
        '#FFB5D6',
        '#FF8DC0',
        '#FFB5D6',
        '#FFE2EE',
        '#73DEFF',
    ]),

    # genderfaun sourced from https://www.flagcolorcodes.com/genderfaun
    'genderfaun': ColorProfile([
        '#FCD689',
        '#FFF09B',
        '#FAF9CD',
        '#FFFFFF',
        '#8EDED9',
        '#8CACDE',
        '#9782EC',
    ]),

    'demifaun': ColorProfile([
        '#7F7F7F',
        '#7F7F7F',
        '#C6C6C6',
        '#C6C6C6',
        '#FCC688',
        '#FFF19C',
        '#FFFFFF',
        '#8DE0D5',
        '#9682EC',
        '#C6C6C6',
        '#C6C6C6',
        '#7F7F7F',
        '#7F7F7F',
    ]),

    # genderfae sourced from https://www.flagcolorcodes.com/genderfae
    'genderfae': ColorProfile([
        '#97C3A5',
        '#C3DEAE',
        '#F9FACD',
        '#FFFFFF',
        '#FCA2C4',
        '#DB8AE4',
        '#A97EDD',
    ]),

    # demifae used colorpicker to source form https://www.deviantart.com/pride-flags/art/Demifae-870194777
    'demifae': ColorProfile([
        '#7F7F7F',
        '#7F7F7F',
        '#C5C5C5',
        '#C5C5C5',
        '#97C3A4',
        '#C4DEAE',
        '#FFFFFF',
        '#FCA2C5',
        '#AB7EDF',
        '#C5C5C5',
        '#C5C5C5',
        '#7F7F7F',
        '#7F7F7F',
    ]),

    'neutrois': ColorProfile([
        '#FFFFFF',
        '#1F9F00',
        '#000000'
    ]),

    'biromantic1': ColorProfile([
        '#8869A5',
        '#D8A7D8',
        '#FFFFFF',
        '#FDB18D',
        '#151638',
    ]),

    'biromantic2': ColorProfile([
        '#740194',
        '#AEB1AA',
        '#FFFFFF',
        '#AEB1AA',
        '#740194',
    ]),

    'autoromantic': ColorProfile([  # symbol interpreted
        '#99D9EA',
        '#99D9EA',
        '#3DA542',
        '#7F7F7F',
        '#7F7F7F',
    ]),

    # i didn't expect this one to work. cool!
    'boyflux2': ColorProfile(ColorProfile([
        '#E48AE4',
        '#9A81B4',
        '#55BFAB',
        '#FFFFFF',
        '#A8A8A8',
        '#81D5EF',
        '#69ABE5',
        '#5276D4',
    ]).with_weights([1, 1, 1, 1, 1, 5, 5, 5])),

    # sourced from https://commons.wikimedia.org/wiki/File:Girlflux_Pride_Flag.jpg
    "girlflux": ColorProfile([
        "f9e6d7",
        "f2526c",
        "bf0311",
        "e9c587",
        "bf0311",
        "f2526c",
        "f9e6d7",
    ]),

    # sourced from https://www.deviantart.com/pride-flags/art/Genderflux-1-543925589
    "genderflux": ColorProfile([
        "f47694",
        "f2a2b9",
        "cecece",
        "7ce0f7",
        "3ecdf9",
        "fff48d",
    ]),
    
    # https://lgbtqia.wiki/wiki/Gendernull
    'nullflux': ColorProfile([
        '#0B0C0E', '#A28DB9', '#E1D4EF', '#F0E6DD', '#665858',
    ]),
    
    'hypergender': ColorProfile([
        "#EFEFEF", "#FFFFFF", "#FBFF75", "#000000", "#FBFF75", "#FFFFFF", "#EFEFEF",
    ]),
    'hyperboy': ColorProfile([
        "#EFEFEF", "#FFFFFF", "#74D7FE", "#000000", "#74D7FE", "#FFFFFF", "#EFEFEF",
    ]),
    'hypergirl': ColorProfile([
        "#EFEFEF", "#FFFFFF", "#FC76D3", "#000000", "#FC76D3", "#FFFFFF", "#EFEFEF",
    ]),
    'hyperandrogyne': ColorProfile([
        "#EFEFEF", "#FFFFFF", "#BB83FF", "#000000", "#BB83FF", "#FFFFFF", "#EFEFEF",
    ]),
    'hyperneutrois': ColorProfile([
        "#EFEFEF", "#FFFFFF", "#BAFA74", "#000000", "#BAFA74", "#FFFFFF", "#EFEFEF",
    ]),

    "finsexual": ColorProfile([
        "#B18EDF",
        "#D7B1E2",
        "#F7CDE9",
        "#F39FCE",
        "#EA7BB3",
    ]),

    'unlabeled1': ColorProfile([
        '#EAF8E4',
        '#FDFDFB',
        '#E1EFF7',
        '#F4E2C4'
    ]),

    'unlabeled2': ColorProfile([
        '#250548',
        '#FFFFFF',
        '#F7DCDA',
        '#EC9BEE',
        '#9541FA',
        '#7D2557'
    ]),

    'pangender': ColorProfile([
        '#FFF798',
        '#FEDDCD',
        '#FFEBFB',
        '#FFFFFF',
        '#FFEBFB',
        '#FEDDCD',
        '#FFF798',
    ]),

    'pangender.contrast': ColorProfile([
        '#ffe87f',
        '#fcbaa6',
        '#fbc9f3',
        '#FFFFFF',
        '#fbc9f3',
        '#fcbaa6',
        '#ffe87f',
    ]),

    'gendernonconforming1': ColorProfile(
        ColorProfile([
            '#50284d',
            '#96467b',
            '#5c96f7',
            '#ffe6f7',
            '#5c96f7',
            '#96467b',
            '#50284d'
        ]).with_weights([
            4, 1, 1, 1, 1, 1, 4
        ])
    ),

    'gendernonconforming2': ColorProfile([
        '#50284d',
        '#96467b',
        '#5c96f7',
        '#ffe6f7',
        '#5c96f7',
        '#96467b',
        '#50284d'
    ]),

    'femboy': ColorProfile([
        "#d260a5",
        "#e4afcd",
        "#fefefe",
        "#57cef8",
        "#fefefe",
        "#e4afcd",
        "#d260a5"
    ]),

    'tomboy': ColorProfile([
        "#2f3fb9",
        "#613a03",
        "#fefefe",
        "#f1a9b7",
        "#fefefe",
        "#613a03",
        "#2f3fb9"
    ]),

    'gynesexual': ColorProfile([
        "#F4A9B7",
        "#903F2B",
        "#5B953B",
    ]),

    'androsexual': ColorProfile([
        "#01CCFF",
        "#603524",
        "#B799DE",
    ]),

    # gendervoid and related flags sourced from: https://gender.fandom.com/wiki/Gendervoid
    'gendervoid': ColorProfile([
        "#081149",
        "#4B484B",
        "#000000",
        "#4B484B",
        "#081149"
    ]),

    'voidgirl': ColorProfile([
        "#180827",
        "#7A5A8B",
        "#E09BED",
        "#7A5A8B",
        "#180827"
    ]),

    'voidboy': ColorProfile([
        "#0B130C",
        "#547655",
        "#66B969",
        "#547655",
        "#0B130C"
    ]),

    # used https://twitter.com/foxbrained/status/1667621855518236674/photo/1 as source and colorpicked
    'nonhuman-unity': ColorProfile([
        "#177B49",
        "#FFFFFF",
        "#593C90"
    ]),

    # used https://pluralpedia.org/w/Plurality#/media/File:Plural-Flag-1.jpg as source and colorpicked
    'plural': ColorProfile([
        "#2D0625",
        "#543475",
        "#7675C3",
        "#89C7B0",
        "#F3EDBD",
    ]),

    # sampled from https://es.m.wikipedia.org/wiki/Archivo:Fraysexual_flag.jpg
    'fraysexual': ColorProfile([
        '#226CB5',
        '#94E7DD',
        '#FFFFFF',
        '#636363',
    ]),

    # Queer Subcultures
    # sourced from https://commons.wikimedia.org/wiki/File:Bear_Brotherhood_flag.svg
    'bear': ColorProfile([
        '#623804',
        '#D56300',
        '#FEDD63',
        '#FEE6B8',
        '#FFFFFF',
        '#555555',
    ]),

    # colorpicked from https://commons.wikimedia.org/wiki/File:Butch_Flag.png
    'butch': ColorProfile([
        '#D72800',
        '#F17623',
        '#FF9C56',
        '#FFFDF6',
        '#FFCE89',
        '#FEAF02',
        '#A37000',
    ]),

    # colorpicked from https://commons.wikimedia.org/wiki/File:Leather,_Latex,_and_BDSM_pride_-_Light.svg
    'leather': ColorProfile([
        '#000000',
        '#252580',
        '#000000',
        '#252580',
        '#FFFFFF',
        '#252580',
        '#000000',
        '#252580',
        '#000000',
    ]),

    # colorpicked from https://commons.wikimedia.org/wiki/File:Official_Otter_Pride_Flag_by_Bearbackgear.jpg
    'otter': ColorProfile([
        '#263881',
        '#5C9DC9',
        '#FFFFFF',
        '#3A291D',
        '#5C9DC9',
        '#263881',
    ]),

    # colorpicked from https://commons.wikimedia.org/wiki/File:Twink_Pride_Flag_(proposed).svg
    'twink': ColorProfile([
        '#FFB2FF',
        '#FFFFFF',
        '#FFFF81',
    ]),
    
    # https://en.wikipedia.org/wiki/File:FatFetishFlag.png
    'adipophilia': ColorProfile(["#000000", "#E16180", "#FFF9BE", "#603E41", "#000000"]),

    'kenochoric': ColorProfile([
        '#000000',
        '#2E1569',
        '#824DB7',
        '#C7A1D6',
    ]),

    'veldian': ColorProfile([
        '#D182A8',
        '#FAF6E0',
        '#69ACBE',
        '#5D448F',
        '#3A113E',
    ]),

    'solian': ColorProfile([
        '#FFF8ED',
        '#FFE7A8',
        '#F1B870',
        '#A56058',
        '#46281E',
    ]),

    'lunian': ColorProfile([
        '#2F0E62',
        '#6F41B1',
        '#889FDF',
        '#7DDFD5',
        '#D2F2E2',
    ]),

    # Start of Extras by Jaida Corvera
    # polyamorous flag colors pulled from https://polyamproud.com/flag
    'polyam': ColorProfile([
        "#FFFFFF",
        "#FCBF00",
        "#009FE3",
        "#E50051",
        "#340C46",
    ]),

    'sapphic': ColorProfile([
        "#FD8BA8",
        "#FBF2FF",
        "#C76BC5",
        "#FDD768",
        "#C76BC5",
        "#FBF2FF",
        "#FD8BA8"
    ]),

    'androgyne': ColorProfile([
        "#FE007F",
        "#9832FF",
        "#00B8E7",
    ]),

    'interprogress': ColorProfile([
        "#FFD800",
        "#7902AA",
        "#FFFFFF",
        "#FFAFC8",
        "#74D7EE",
        "#613915",
        "#000000",
        '#E50000',
        '#FF8D00',
        '#FFEE00',
        '#028121',
        '#004CFF',
        '#770088'
    ]),

    'progress': ColorProfile([
        "#FFFFFF",
        "#FFAFC8",
        "#74D7EE",
        "#613915",
        "#000000",
        '#E50000',
        '#FF8D00',
        '#FFEE00',
        '#028121',
        '#004CFF',
        '#770088'
    ]),

    'intersex': ColorProfile([
        "#FFD800",
        "#FFD800",
        "#7902AA",
        "#FFD800",
        "#FFD800"
    ]),

    'old-polyam': ColorProfile([
        "#0000FF",
        "#FF0000",
        "#FFFF00",
        "#FF0000",
        "#000000"
    ]),

    'equal-rights': ColorProfile([
        "#0000FF",
        "#0000FF",
        "#FFFF00",
        "#0000FF",
        "#0000FF",
        "#FFFF00",
        "#0000FF",
        "#0000FF"
    ]),

    'drag': ColorProfile([
        "#CC67FF",
        "#FFFFFF",
        "#FFA3E3",
        "#FFFFFF",
        "#3366FF"
    ]),

    # Pronoun Flags

    'pronounfluid': ColorProfile([
        "#ffb3f9",
        "#ffffff",
        "#d1fdcb",
        "#c7b0ff",
        "#000000",
        "#b8ccff"
    ]),

    'pronounflux': ColorProfile([
        "#fdb3f8",
        "#b6ccfa",
        "#18ddd3",
        "#64ff89",
        "#ff7690",
        "#ffffff"
    ]),

    'exipronoun': ColorProfile([
        "#1c3d34",
        "#ffffff",
        "#321848",
        "#000000"
    ]),

    'neopronoun': ColorProfile([
        "#bcec64",
        "#ffffff",
        "#38077a"
    ]),

    'neofluid': ColorProfile([
        "#ffeca0",
        "#ffffff",
        "#ffeca0",
        "#38087a",
        "#bcec64"
    ]),

    'genderqueer': ColorProfile([
        "#b57edc",
        "#b57edc",
        "#ffffff",
        "#ffffff",
        "#4a8123",
        "#4a8123"
    ]),

    'cisgender': ColorProfile([
        "#D70270",
        "#0038A7"
    ]),

    # colors from Gilbert Baker's original 1978 flag design
    # used https://gilbertbaker.com/rainbow-flag-color-meanings/ as source and colorpicked
    'baker': ColorProfile([
        '#F23D9E',
        '#F80A24',
        '#F78022',
        '#F9E81F',
        '#1E972E',
        '#1B86BC',
        '#243897',
        '#6F0A82',
    ]),

    # this is 4 all the dogs, from zombpawcoins on tumblr!
    'caninekin': ColorProfile([
        '#2d2822',
        '#543d25',
        '#9c754d',
        '#e8dac2',
        '#cfad8c',
        '#b77b55',
        '#954e31'
    ]),

    # Meme flags
    'beiyang': ColorProfile([
        '#DF1B12',
        '#FFC600',
        '#01639D',
        '#FFFFFF',
        '#000000',
    ]),

    'burger': ColorProfile([
        '#F3A26A',
        '#498701',
        '#FD1C13',
        '#7D3829',
        '#F3A26A',
    ]),

    'throatlozenges': ColorProfile([
        "#2759DA",
        "#03940D",
        "#F5F100",
        "#F59B00",
        "#B71212"
    ]),

    'band': ColorProfile([
        "#2670c0",
        "#f5bd00",
        "#dc0045",
        "#e0608e"
    ]),
}



import json
from pathlib import Path

import requests

if __name__ == '__main__':
    md = requests.get("https://raw.githubusercontent.com/lukas-w/font-logos/refs/heads/master/README.md").text
    md = [[field.strip().strip('`').replace('Linux', '').replace('GNU/', '').strip() for field in line.split('|')[1:-1]]
          for line in md.splitlines() if line.startswith("|") and 'fl-' in line]
    md = {name: chr(int(char, 16)) for name, css, code, char, img in md}
    Path("font_logos.json").write_text(json.dumps(md, indent=2, ensure_ascii=False))



# This file is automatically generated. Please do not modify.

from . import AsciiArt

opensuse_slowroll = AsciiArt(match=r'''"openSUSE Slowroll"* | "openSUSE_Slowroll"*''', color='2 7', ascii=r"""
${c2}              _aaaymQQmwaaa,
          ,wWQQQD????????$QQQQa,.
       _wQQB?"              ??QQQa,
     sQQD^                      ?QQ6\
    yWW'                          4QQg
  ,QQD          .aaaaaaaa          ^4Q6
 ,mQP        _wWQW?????YWWQa,        4Qm
 jQ@        wWW?'        ^4QQc       ^$QL
,QQ'       jWW'            )QW\       ]QQ
|QQ       ,QW'              ]QQ       ^QQ|
|QQ       |QQ               ]QQ        QQ|
|QQ        4Qg              ]QQ       .QQ|
'QQ6       '$WQac.         _QQ(       jQQ
 ]QQw        "?QWQQf      _mQP       ,QQ(
  4QQga                  wQQP       ,mQ?
   4QQQga,            saQWP'       jQQf
    ?QQQQQQwaaaaaaaayWWW?'       _mQ@'
      ?WQQQP?9VWUV???^        _amQP^
        "4QQQaa,          ,awQQQ?^
           "?VQQQQQQQQQQQQQQP?'
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

fedora_sericea = AsciiArt(match=r'''"Fedora Sericea"* | "Sericea"*''', color='12 7', ascii=r"""
${c1}              :oooo,  .','
      .';;;.;oooooooolooooo'
     coooooooooooooooooooooooolc'
  .':oooooooooooo${c2}ll${c1}ooooooooooooool
.oooooooooooooooo${c2}ll${c1}oooooooooooo${c2}l${c1}ool
ooooooooooooooooo${c2}ll${c1}ooooooooooo${c2}ll${c1}oo'
 oooo${c2}l${c1}oooooooooo${c2}lll${c1}ooooooooo${c2}lll${c1}oo
 .ooooo${c2}lll${c1}ooooo${c2}lll${c1}ooooooooo${c2}lll${c1}ool
 .ooooooo${c2}lll${c1}oo${c2}llll${c1}oooo${c2}lllll${c1}ooooo:
  'oooooooo${c2}llllllllllll${c1}oooooooo'
     .c,.oo${c2}lllll${c1}oooooooo.${c2}
           'll;
           'll.
           lll
           lll
          ;ll,
          .l:
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

kaos = AsciiArt(match=r'''"KaOS"*''', color='4 7 1', ascii=r"""
${c1}                     ..
  .....         ..OSSAAAAAAA..
 .KKKKSS.     .SSAAAAAAAAAAA.
.KKKKKSO.    .SAAAAAAAAAA...
KKKKKKS.   .OAAAAAAAA.
KKKKKKS.  .OAAAAAA.
KKKKKKS. .SSAA..
.KKKKKS..OAAAAAAAAAAAA........
 DKKKKO.=AA=========A===AASSSO..
  AKKKS.==========AASSSSAAAAAASS.
  .=KKO..========ASS.....SSSSASSSS.
    .KK.       .ASS..O.. =SSSSAOSS:
     .OK.      .ASSSSSSSO...=A.SSA.
       .K      ..SSSASSSS.. ..SSA.
                 .SSS.AAKAKSSKA.
                    .SSS....S..
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

haiku = AsciiArt(match=r'''"Haiku"*''', color='1 3 7 2', ascii=r'''
${c3}

           MMMM              MMMM
           MMMM              MMMM
           MMMM              MMMM
           MMMM              MMMM
           MMMM${c4}       .ciO| /YMMMMM*"
${c3}           MMMM${c4}   .cOMMMMM|/MMMMM/`
 ,         ,iMM|/MMMMMMMMMMMMMMM*
  `*.__,-cMMMMMMMMMMMMMMMMM/`${c3}.MMM
           MM${c4}MMMMMMM/`:MMM/  ${c3}MMMM
           MMMM              MMMM
           MMMM              MMMM
           """"              """"
''')
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

void_small = AsciiArt(match=r'''"void_small"''', color='2 8', ascii=r"""
${c1}    _______
 _ \______ -
| \  ___  \ |
| | /   \ | |
| | \___/ | |
| \______ \_|
 -_______\
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

panwah = AsciiArt(match=r'''"Panwah"*''', color='7 1 0', ascii=r"""
${c1}
          HHH
         HAAAH                             HHH
         HAAAAH                           HAAAH
        HAAAAAAH                         HAAAAH
        HAAAAAAH                       HAAAAAH
       HAAAAAAAAH${c2}WWWWWWWWWWWWWWWW      ${c1}HAAAAAH$
       HAAAAAAAAH${c2}WWWWWWWWWWWWWWWWWWWW${c1} HAAAAAH
       HAA${c2}WWWWWWWWWWWWWWWWWWWWWWWWWWWWW${c1}AAAAAH${c2}
      WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW${c1}WAH${c2}
     WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW
   WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW
  WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW
 WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW
 WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW
 WWWWWWW${c1}AAA${c2}WWWW   WWWWWWWWWWWWWWWWWWWWWWWWWWW
   WWWW${c1}AAA${c2}WWWWW    WWWWWWW    WWWWWWWWWWWWWWW
    WW${c1}AAA${c2}WWWWWWWWWWWWWWWWW   WWWWW${c1}AAA${c2}WWWWWWWW
     ${c1}AAA${c2}WWWWW${c1}OOOOOOOOOOO${C2}WWWWWWWWWWW${c1}AAA${c2}WWWWWW
           ${c1}OOOO${c3}GGGGGGG${c1}OOOO${c2}WWWWWWWWWW${c1}AAA${c2}WWWW
            ${c1}OOO${c3}GGGGGGG${c1}OOO${c2}WWWWWWWWWWWW${c1}AAA${c2}W
              ${c1}OOOOOOOOO
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

slackel = AsciiArt(match=r'''"Slackel"*''', color='3', ascii=r'''
${c1}              _aawmmmmmwwaaaaas,,,_.
           .ammmmm###mmmmmmm###BQmm##mws
         .am###mmBmBmBmBmBmBmmmmm#mmmm#2
        <q###mmBmBmBmBmBmBmBmBmBmBmmBmZ`
       um#mmmBmBm##U##mmBmBmBmWmmBmWm#(
     .wm#mmBBmm#Y~   ~XmBmBmWmmmmmBm#e
    .dm#mmWmm#Z'      ]#mBmBmmBZ!""""`
   .dm#mmBmm#2`       ]mmmBmBm#2
   jm#mmWmm#2`        dmmBmBmB#(
  )m##mBmmWZ`        )##mBmBmmZ
 :dmmmBmBm#'        .d#mBmBmWZ(
 j#mmBmBmme         jmmmBmBm#2
_m#mBmWmmm'        )mmmBmBmmZ`
]##mBmmm#2        <m#mBmBmB#^
dmmmBmWm#C       <m#mBmBmB#(
ZmmBmBmmmh.    _jm#mmBmBm#(
XBmBmBmBmm6s_aum##mmBmBm&^
3$mBmBmBmmm#mmmmmmBmBm#2'
+ZmBmBmWmBmBmWmmBmBm##!
 )ZmBmBmmmBmBmmBmB##!`
  -4U#mBmWmBmBm##2"
    -!!XU##US*?"-
''')
    


from __future__ import annotations

from dataclasses import dataclass, field

from .constants import CONFIG_PATH
from .neofetch_util import ColorAlignment
from .serializer import json_stringify, from_dict
from .types import AnsiMode, LightDark, BackendLiteral


@dataclass
class Config:
    preset: str
    mode: AnsiMode
    light_dark: LightDark = 'dark'
    lightness: float | None = None
    color_align: ColorAlignment = field(default_factory=lambda: ColorAlignment('horizontal'))
    backend: BackendLiteral = "neofetch"
    args: str | None = None
    distro: str | None = None
    pride_month_shown: list[int] = field(default_factory=list)  # This is deprecated, see issue #136
    pride_month_disable: bool = False

    @classmethod
    def from_dict(cls, d: dict):
        d['color_align'] = ColorAlignment.from_dict(d['color_align'])
        return from_dict(cls, d)

    def save(self):
        CONFIG_PATH.parent.mkdir(exist_ok=True, parents=True)
        CONFIG_PATH.write_text(json_stringify(self, indent=4), 'utf-8')



# This file is automatically generated. Please do not modify.

from . import AsciiArt

archlabs = AsciiArt(match=r'''"ARCHlabs"*''', color='6 6 7 1', ascii=r"""
${c1}                     'c'
                    'kKk,
                   .dKKKx.
                  .oKXKXKd.
                 .l0XXXXKKo.
                 c0KXXXXKX0l.
                :0XKKOxxOKX0l.
               :OXKOc. .c0XX0l.
              :OK0o. ${c4}...${c1}'dKKX0l.
             :OX0c  ${c4};xOx'${c1}'dKXX0l.
            :0KKo.${c4}.o0XXKd'.${c1}lKXX0l.
           c0XKd.${c4}.oKXXXXKd..${c1}oKKX0l.
         .c0XKk;${c4}.l0K0OO0XKd..${c1}oKXXKo.
        .l0XXXk:${c4},dKx,.'l0XKo.${c1}.kXXXKo.
       .o0XXXX0d,${c4}:x;   .oKKx'${c1}.dXKXXKd.
      .oKXXXXKK0c.${c4};.    :00c'${c1}cOXXXXXKd.
     .dKXXXXXXXXk,${c4}.     cKx'${c1}'xKXXXXXXKx'
    'xKXXXXK0kdl:.     ${c4}.ok; ${c1}.cdk0KKXXXKx'
   'xKK0koc,..         ${c4}'c, ${c1}    ..,cok0KKk,
  ,xko:'.             ${c4}.. ${c1}           .':okx;
 .,'.                                   .',.
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

hyperbola = AsciiArt(match=r'''"Hyperbola"*''', color='8', ascii=r"""
${c1}                     WW
                     KX              W
                    WO0W          NX0O
                    NOO0NW  WNXK0OOKW
                    W0OOOOOOOOOOOOKN
                     N0OOOOOOO0KXW
                       WNXXXNW
                 NXK00000KN
             WNK0OOOOOOOOOO0W
           NK0OOOOOOOOOOOOOO0W
         X0OOOOOOO00KK00OOOOOK
       X0OOOO0KNWW      WX0OO0W
     X0OO0XNW              KOOW
   N00KNW                   KOW
 NKXN                       W0W
WW                           W
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

biglinux_large = AsciiArt(match=r'''"BigLinux_large"*''', color='2 3 4 7', ascii=r"""
${c4}
                                              ${c3}.cd0KXXX${c4}0${c3}x;
                                           ${c3}.oXM${c4}M${c3}MMMMMMMMMWo
                             ${c2}...          ${c3}dWMMMM${c4}M${c3}MMMMMMMMMMN,
                          ${c2}c0WMMMW0:     ${c3}.XMMMM${c4}M${c3}MMMKdc;,;cxNMM:
    ${c1}x,                  ${c2}.XMMMMMMMMM:   ${c3}'W${c3}MMMMMMWo.         ;KW.
   ${c1}dM.                  ${c2}.WMMMMMMMM0    ${c3}N${c3}MMMM${c4}M${c3}Mk.             dd
  ${c1}.MM.                   ${c2}.lOKXKOl.    ${c3}oMMMMMMd         .l${c4}x${c3}kd: .
  ${c1}kMM:          .'..         ${c2}:xOOxc.  ${c3}O${c4}M${c3}MMMM0        lXMMMMM${c4}M${c3}N;
  ${c1}WMM0       lKMMMMMW0o.    ${c2}KMMMMMMN. ${c3}xMM${c4}M${c3}M,     .dWMMMMMMMMMM;
 ${c1}.MMMMx   .oWMMMMMMOl0MMX;  ${c2}NMMMMMMM; ${c3}.NMMMM.   .dWMMMM${c4}M${c3}MMMMMMMN
 ${c1}.MMMMMNOKMMMMMMMk.  'MMMMx ${c2};MMMMMMMO  ${c3}.KMMMl .oWMMMMMWk:,lWMMMM.
  ${c1}xMMMMMMMMMMMMk.    .MMMMMc ${c2}'XMMMMMMx   ${c3}:0MMNMMMM${c4}M${c3}MWx.    .WM${c4}M${c3}M.
   ${c1}0MMMMMMMMWx.      :MMMMMK   ${c2}cKMMMMMX:    ${c3},ok0K0x:.       oMMN
    ${c1}oNMMMM0c.       .NMMMMMO     ${c2}.lOWMMMX:                  ${c3}.M${c4}M${c3}o
   ${c1}c. .'.          .KMMMMMM:         ${c2}.';clc.                 ${c3}MN
   ${c1}:N;            cWMMMMMMO                                 ${c3}.W;
    ${c1}0M0:       'dNMMMMMMM0                                  ${c3}',
     ${c1}0MMMX0O0XMMMMMMMMMMo
      ${c1}oWMMMMMMMMMMMMMMk.
       ${c1}.oXMMMMMMMMW0c.
          ${c1}.;ccc:,.
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

elementary_small = AsciiArt(match=r'''"elementary_small"*''', color='4 7 1', ascii=r"""
${c2}  _______
 / ____  \
/  |  /  /\
|__\ /  / |
\   /__/  /
 \_______/
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

irix = AsciiArt(match=r'''"IRIX"*''', color='4 7', ascii=r"""
${c1}           ./ohmNd/  +dNmho/-
     `:+ydNMMMMMMMM.-MMMMMMMMMdyo:.
   `hMMMMMMNhs/sMMM-:MMM+/shNMMMMMMh`
   -NMMMMMmo-` /MMM-/MMM- `-omMMMMMN.
 `.`-+hNMMMMMNhyMMM-/MMMshmMMMMMmy+...`
+mMNds:-:sdNMMMMMMMyyMMMMMMMNdo:.:sdMMm+
dMMMMMMmy+.-/ymNMMMMMMMMNmy/-.+hmMMMMMMd
oMMMMmMMMMNds:.+MMMmmMMN/.-odNMMMMmMMMM+
.MMMM-/ymMMMMMmNMMy..hMMNmMMMMMmy/-MMMM.
 hMMM/ `/dMMMMMMMN////NMMMMMMMd/. /MMMh
 /MMMdhmMMMmyyMMMMMMMMMMMMhymMMMmhdMMM:
 `mMMMMNho//sdMMMMM//NMMMMms//ohNMMMMd
  `/so/:+ymMMMNMMMM` mMMMMMMMmh+::+o/`
     `yNMMNho-yMMMM` NMMMm.+hNMMNh`
     -MMMMd:  oMMMM. NMMMh  :hMMMM-
      -yNMMMmooMMMM- NMMMyomMMMNy-
        .omMMMMMMMM-`NMMMMMMMmo.
          `:hMMMMMM. NMMMMMh/`
             .odNm+  /dNms.
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

proxmox = AsciiArt(match=r'''"Proxmox"*''', color='7 202', ascii=r"""
${c1}         .://:`              `://:.
       `hMMMMMMd/          /dMMMMMMh`
        `sMMMMMMMd:      :mMMMMMMMs`
${c2}`-/+oo+/:${c1}`.yMMMMMMMh-  -hMMMMMMMy.`${c2}:/+oo+/-`
`:oooooooo/${c1}`-hMMMMMMMyyMMMMMMMh-`${c2}/oooooooo:`
  `/oooooooo:${c1}`:mMMMMMMMMMMMMm:`${c2}:oooooooo/`
    ./ooooooo+-${c1} +NMMMMMMMMN+ ${c2}-+ooooooo/.
      .+ooooooo+-${c1}`oNMMMMNo`${c2}-+ooooooo+.
        -+ooooooo/.${c1}`sMMs`${c2}./ooooooo+-
          :oooooooo/${c1}`..`${c2}/oooooooo:
          :oooooooo/`${c1}..${c2}`/oooooooo:
        -+ooooooo/.`${c1}sMMs${c2}`./ooooooo+-
      .+ooooooo+-`${c1}oNMMMMNo${c2}`-+ooooooo+.
    ./ooooooo+-${c1} +NMMMMMMMMN+ ${c2}-+ooooooo/.
  `/oooooooo:`${c1}:mMMMMMMMMMMMMm:${c2}`:oooooooo/`
`:oooooooo/`${c1}-hMMMMMMMyyMMMMMMMh-${c2}`/oooooooo:`
`-/+oo+/:`${c1}.yMMMMMMMh-  -hMMMMMMMy.${c2}`:/+oo+/-`
${c1}        `sMMMMMMMm:      :dMMMMMMMs`
       `hMMMMMMd/          /dMMMMMMh`
         `://:`              `://:`
""")
    


from __future__ import annotations

import string


class AsciiArt:
    name: str
    match: str
    color: str
    ascii: str

    def __init__(self, match: str, color: str, ascii: str, name: str | None = None):
        self.match = match
        self.color = color
        self.ascii = ascii
        self.name = name or self.get_friendly_name()

    def get_friendly_name(self) -> str:
        return self.match.split("|")[0].strip(string.punctuation + '* ') \
            .replace('"', '').replace('*', '')



# This file is automatically generated. Please do not modify.

from . import AsciiArt

opensuse_tumbleweed = AsciiArt(match=r'''"openSUSE Tumbleweed"* | "openSUSE_Tumbleweed"*''', color='2 7', ascii=r"""
${c2}         JJJJJJJJ
      JJJJJJJJJJJJJJ
    JJJJJJ   =JJJJJJJ
   JJJJ      =JJJ JJJJ
   JJJ       =JJJ   JJJ
  JJJJ       =JJJ   JJJ
  JJJJJJJJJJJJJJJ   JJJJ
   JJJJJJJJJJJJJJ   JJJJ
   JJJJ             JJJJ
    JJJJJ=          JJJJ
      JJJJJJJJJJJJJJJJJJJJJJJJJJJJJ=
        =JJJJJJJJJJJJJJJJJJJJJJJJJJJJJ
                    JJJJ         =JJJJJJ
                    JJJJ            =JJJJ
                    JJJJ   JJJJJJJJJJJJJJ
                    JJJJ   JJJJJJJJJJJJJJJ
                    JJJJ   JJJJ       JJJJ
                     JJJ   JJJJ       JJJ
                     JJJJJ JJJJ      JJJJ
                      =JJJJJJJJ   JJJJJJ
                        JJJJJJJJJJJJJJ
                           JJJJJJJ=
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

celos = AsciiArt(match=r'''"CelOS"*''', color='4 6 0 5', ascii=r"""

${c4}                     .,cmmmmmmmmmmmc,.
                .,cmMMMMMMMMMMMMMMMMMMMMmc.
             .cMMMMMMMMMMMMMMMMMMMMMMMMMMMmc.
           .cMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMc.
         ,:MMM ${c3}####################################${c4}
        cMMMMMMmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmc.
       .MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM.
      .MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMc
      "******************************MMMMMMMMMMMMMc:
${c3}#################################### ${c4}MMMMMMMMMMMMMc
      "MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM:
       "MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM"
       'MMMMMMMMM*******************************:
        \"MMMMMM ${c3}#####################################
         ${c4}`:MMMMMMmmmmmmmmmmmmmmmmmmmmmmmmmmmmm;
           `"MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM"
             `":MMMMMMMMMMMMMMMMMMMMMMMMM;'
                `":MMMMMMMMMMMMMMMMMMM:"
                     "************"
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

endless = AsciiArt(match=r'''"Endless"*''', color='1 7', ascii=r"""
${c1}           `:+yhmNMMMMNmhy+:`
        -odMMNhso//////oshNMMdo-
      /dMMh+.              .+hMMd/
    /mMNo`                    `oNMm:
  `yMMo`                        `oMMy`
 `dMN-                            -NMd`
 hMN.                              .NMh
/MM/                  -os`          /MM/
dMm    `smNmmhs/- `:sNMd+   ``       mMd
MMy    oMd--:+yMMMMMNo.:ohmMMMNy`    yMM
MMy    -NNyyhmMNh+oNMMMMMy:.  dMo    yMM
dMm     `/++/-``/yNNh+/sdNMNddMm-    mMd
/MM/          `dNy:       `-::-     /MM/
 hMN.                              .NMh
 `dMN-                            -NMd`
  `yMMo`                        `oMMy`
    /mMNo`                    `oNMm/
      /dMMh+.              .+hMMd/
        -odMMNhso//////oshNMMdo-
           `:+yhmNMMMMNmhy+:`
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

openmamba = AsciiArt(match=r'''"openmamba"*''', color='7 2', ascii=r"""
${c1}                 `````
           .-/+ooooooooo+/:-`
        ./ooooooooooooooooooo+:.
      -+oooooooooooooooooooooooo+-
    .+ooooooooo+/:---::/+ooooooooo+.
   :oooooooo/-`          `-/oo${c2}s´${c1}oooo.${c2}s´${c1}
  :ooooooo/`                `${c2}sNds${c1}ooo${c2}sNds${c1}
 -ooooooo-                   ${c2}:dmy${c1}ooo${c2}:dmy${c1}
 +oooooo:                      :oooooo-
.ooooooo                        .://:`
:oooooo+                        ./+o+:`
-ooooooo`                      `oooooo+
`ooooooo:                      /oooooo+
 -ooooooo:                    :ooooooo.
  :ooooooo+.                .+ooooooo:
   :oooooooo+-`          `-+oooooooo:
    .+ooooooooo+/::::://oooooooooo+.
      -+oooooooooooooooooooooooo+-
        .:ooooooooooooooooooo+:.
           `-:/ooooooooo+/:.`
                 ``````
""")
    


#!/usr/bin/env python3
import argparse
import json
import os
import shlex
from subprocess import check_output

import pyperclip
import requests
from github import Github

upstream = 'dylanaraps/neofetch'
my_fork = 'hykilpikonna/hyfetch'
my_base = 'master'

http = requests.Session()
if 'GH_TOKEN' in os.environ:
    print('Token loaded')
    http.headers['Authorization'] = f'token {os.environ["GH_TOKEN"]}'


def copy_comment():
    # Get commit SHA
    sha = check_output(shlex.split('git rev-parse --short HEAD')).decode().strip()

    # Copy comment to clipboard
    comment = f"""
Thank you for your contribution! 

This PR is [merged into hyfetch](https://github.com/hykilpikonna/hyfetch/commit/{sha}) since this repo (dylanaraps/neofetch) seems no longer maintained.

[HyFetch](https://github.com/hykilpikonna/hyfetch) is a fork of neofetch with LGBTQ pride flags, but the repo also maintains an updated version of the original neofetch, addressing many pull requests that are not merged in the original repo.

Read the ["Running Updated Original Neofetch" section](https://github.com/hykilpikonna/hyfetch#running-updated-original-neofetch) for more info!
    """
    pyperclip.copy(comment.strip())
    print()
    print('Done!')
    print('Comment response copied to clipboard.')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Helper for accepting upstream pull requests')
    parser.add_argument('pull', type=int, help='Pull request number')
    args = parser.parse_args()
    pr = args.pull

    print(f'Accepting pull request {pr}...')

    # Fetch original pr's information
    info = http.get(f'https://api.github.com/repos/{upstream}/pulls/{pr}').json()
    user = info['user']['login']

    # Fetch commit information
    commits = http.get(f'https://api.github.com/repos/{upstream}/pulls/{pr}/commits').json()
    override_author = os.environ.get("override_author")
    author = json.loads(override_author) if override_author else commits[-1]['commit']['author']

    # Create commit message
    title = info["title"].replace('"', '\\"')
    msg = (f'-m "[PR] {upstream}#{pr} from {user} - {title}" '
           f'-m "Upstream PR: https://github.com/{upstream}/pull/{pr}  \n'
           f'Thanks to @{user}\n\n'
           f'Co-authored-by: {author["name"]} <{author["email"]}>"')

    # head could be null, if the pr repo is deleted
    if info['head'] is None or info['head']['repo'] is None:
        print(f'Original repo is deleted. Please manually merge.')
        input('Press any key to continue when the changes are made...')

        # Commit with merge
        print()
        print('Committing merge...')
        os.system(f'git commit -a {msg}')

    # Automatically merge
    else:
        head = info['head']['repo']['full_name']
        head_br = info['head']['ref']
        head_lbl = info['head']['label']
        print()
        print('Original Pull Request Info:')
        print('> State:', info['state'])
        print('> Title:', info['title'])
        print('> User:', user)
        print('> Created:', info['created_at'])
        print('> Head:', head, head_br, head_lbl)

        # Fetch head branch
        print()
        print('Fetching head branch...')
        os.system(f'git fetch https://github.com/{head} {head_br}')

        # Merge head branch
        print()
        print('Merging fetch_head...')
        os.system(f'git merge FETCH_HEAD --no-ff --no-edit {msg}')

    # Push
    print()
    assert input('Push? [Enter/N]') == ""
    os.system('git push')

    copy_comment()



# This file is automatically generated. Please do not modify.

from . import AsciiArt

arch_small = AsciiArt(match=r'''"arch_small"''', color='6 7 1', ascii=r"""
${c1}      /\
     /  \
    /\   \
${c2}   /      \
  /   ,,   \
 /   |  |  -\
/_-''    ''-_\
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

artix_small = AsciiArt(match=r'''"artix_small"*''', color='6 6 7 1', ascii=r"""
${c1}            '
           'A'
          'ooo'
         'ookxo'
         `ookxxo'
       '.   `ooko'
      'ooo`.   `oo'
     'ooxxxoo`.   `'
    'ookxxxkooo.`   .
   'ookxxkoo'`   .'oo'
  'ooxoo'`     .:ooxxo'
 'io'`             `'oo'
'`                     `'
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

lubuntu = AsciiArt(match=r'''"Lubuntu"*''', color='4 7 1', ascii=r"""
${c1}           `.:/ossyyyysso/:.
        `.:yyyyyyyyyyyyyyyyyy:.`
      .:yyyyyyyyyyyyyyyyyyyyyyyy:.
    .:yyyyyyyyyyyyyyyyyyyyyyyyyyyy:.
   -yyyyyyyyyyyyyy${c2}+hNMMMNh+${c1}yyyyyyyyy-
  :yy${c2}mNy+${c1}yyyyyyyy${c2}+Nmso++smMdhyysoo+${c1}yy:
 -yy${c2}+MMMmmy${c1}yyyyyy${c2}hh${c1}yyyyyyyyyyyyyyyyyyy-
.yyyy${c2}NMN${c1}yy${c2}shhs${c1}yyy${c2}+o${c1}yyyyyyyyyyyyyyyyyyyy.
:yyyy${c2}oNM+${c1}yyyy${c2}+sso${c1}yyyyyyy${c2}ss${c1}yyyyyyyyyyyyy:
:yyyyy${c2}+dNs${c1}yyyyyyy${c2}++${c1}yyyyy${c2}oN+${c1}yyyyyyyyyyyy:
:yyyyy${c2}oMMmhysso${c1}yyyyyyyyyy${c2}mN+${c1}yyyyyyyyyyy:
:yyyyyy${c2}hMm${c1}yyyyy${c2}+++${c1}yyyyyyy${c2}+MN${c1}yyyyyyyyyyy:
.yyyyyyy${c2}ohmy+${c1}yyyyyyyyyyyyy${c2}NMh${c1}yyyyyyyyyy.
 -yyyyyyyyyy${c2}++${c1}yyyyyyyyyyyy${c2}MMh${c1}yyyyyyyyy-
  :yyyyyyyyyyyyyyyyyyyyy${c2}+mMN+${c1}yyyyyyyy:
   -yyyyyyyyyyyyyyyyy${c2}+sdMMd+${c1}yyyyyyyy-
    .:yyyyyyyyy${c2}hmdmmNMNdy+${c1}yyyyyyyy:.
      .:yyyyyyy${c2}my${c1}yyyyyyyyyyyyyyy:.
        `.:yyyy${c2}s${c1}yyyyyyyyyyyyy:.`
           `.:/oosyyyysso/:.`
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

clover = AsciiArt(match=r'''"Clover"*''', color='2 6', ascii=r"""
${c1}               `omo``omo`
             `oNMMMNNMMMNo`
           `oNMMMMMMMMMMMMNo`
          oNMMMMMMMMMMMMMMMMNo
          `sNMMMMMMMMMMMMMMNs`
     `omo`  `sNMMMMMMMMMMNs`  `omo`
   `oNMMMNo`  `sNMMMMMMNs`  `oNMMMNo`
 `oNMMMMMMMNo`  `oNMMNs`  `oNMMMMMMMNo`
oNMMMMMMMMMMMNo`  `sy`  `oNMMMMMMMMMMMNo
`sNMMMMMMMMMMMMNo.${c2}oNNs${c1}.oNMMMMMMMMMMMMNs`
`oNMMMMMMMMMMMMNs.${c2}oNNs${c1}.oNMMMMMMMMMMMMNo`
oNMMMMMMMMMMMNs`  `sy`  `oNMMMMMMMMMMMNo
 `oNMMMMMMMNs`  `oNMMNo`  `oNMMMMMMMNs`
   `oNMMMNs`  `sNMMMMMMNs`  `oNMMMNs`
     `oNs`  `sNMMMMMMMMMMNs`  `oNs`
          `sNMMMMMMMMMMMMMMNs`
          +NMMMMMMMMMMMMMMMMNo
           `oNMMMMMMMMMMMMNo`
             `oNMMMNNMMMNs`
               `omo``oNs`
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

crux = AsciiArt(match=r'''"CRUX"*''', color='4 5 7 6', ascii=r"""
${c1}         odddd
      oddxkkkxxdoo
     ddcoddxxxdoool
     xdclodod  olol
     xoc  xdd  olol
     xdc  ${c2}k00${c1}Okdlol
     xxd${c2}kOKKKOkd${c1}ldd
     xdco${c2}xOkdlo${c1}dldd
     ddc:cl${c2}lll${c1}oooodo
   odxxdd${c3}xkO000kx${c1}ooxdo
  oxdd${c3}x0NMMMMMMWW0od${c1}kkxo
 oooxd${c3}0WMMMMMMMMMW0o${c1}dxkx
docldkXW${c3}MMMMMMMWWN${c1}Odolco
xx${c2}dx${c1}kxxOKN${c3}WMMWN${c1}0xdoxo::c
${c2}xOkkO${c1}0oo${c3}odOW${c2}WW${c1}XkdodOxc:l
${c2}dkkkxkkk${c3}OKX${c2}NNNX0Oxx${c1}xc:cd
${c2} odxxdx${c3}xllod${c2}ddooxx${c1}dc:ldo
${c2}   lodd${c1}dolccc${c2}ccox${c1}xoloo
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

haiku_small = AsciiArt(match=r'''"haiku_small"*''', color='2 8', ascii=r"""
${c1}       ,^,
      /   \
*--_ ;     ; _--*
\   '"     "'   /
 '.           .'
.-'"         "'-.
 '-.__.   .__.-'
       |_|
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

exherbo = AsciiArt(match=r'''"Exherbo"*''', color='4 7 1', ascii=r"""
${c2} ,
OXo.
NXdX0:    .cok0KXNNXXK0ko:.
KX  '0XdKMMK;.xMMMk, .0MMMMMXx;  ...
'NO..xWkMMx   kMMM    cMMMMMX,NMWOxOXd.
  cNMk  NK    .oXM.   OMMMMO. 0MMNo  kW.
  lMc   o:       .,   .oKNk;   ;NMMWlxW'
 ;Mc    ..   .,,'    .0M${c1}g;${c2}WMN'dWMMMMMMO
 XX        ,WMMMMW.  cM${c1}cfli${c2}WMKlo.   .kMk
.Mo        .WM${c1}GD${c2}MW.   XM${c1}WO0${c2}MMk        oMl
,M:         ,XMMWx::,''oOK0x;          NM.
'Ml      ,kNKOxxxxxkkO0XXKOd:.         oMk
 NK    .0Nxc${c3}:::::::::::::::${c2}fkKNk,      .MW
 ,Mo  .NXc${c3}::${c2}qXWXb${c3}::::::::::${c2}oo${c3}::${c2}lNK.    .MW
  ;Wo oMd${c3}:::${c2}oNMNP${c3}::::::::${c2}oWMMMx${c3}:${c2}c0M;   lMO
   'NO;W0c${c3}:::::::::::::::${c2}dMMMMO${c3}::${c2}lMk  .WM'
     xWONXdc${c3}::::::::::::::${c2}oOOo${c3}::${c2}lXN. ,WMd
      'KWWNXXK0Okxxo,${c3}:::::::${c2},lkKNo  xMMO
        :XMNxl,';:lodxkOO000Oxc. .oWMMo
          'dXMMXkl;,.        .,o0MMNo'
             ':d0XWMMMMWNNNNMMMNOl'
                   ':okKXWNKkl'
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

libreelec = AsciiArt(match=r'''"LibreELEC"*''', color='2 3 7 14 13', ascii=r"""
${c1}          :+ooo/.      ${c2}./ooo+:
${c1}        :+ooooooo/.  ${c2}./ooooooo+:
${c1}      :+ooooooooooo:${c2}:ooooooooooo+:
${c1}    :+ooooooooooo+-  ${c2}-+ooooooooooo+:
${c1}  :+ooooooooooo+-  ${c3}--  ${c2}-+ooooooooooo+:
${c1}.+ooooooooooo+-  ${c3}:+oo+:  ${c2}-+ooooooooooo+-
${c1}-+ooooooooo+-  ${c3}:+oooooo+:  ${c2}-+oooooooooo-
${c1}  :+ooooo+-  ${c3}:+oooooooooo+:  ${c2}-+oooooo:
${c1}    :+o+-  ${c3}:+oooooooooooooo+:  ${c2}-+oo:
${c4}     ./   ${c3}:oooooooooooooooooo:   ${c5}/.
${c4}   ./oo+:  ${c3}-+oooooooooooooo+-  ${c5}:+oo/.
${c4} ./oooooo+:  ${c3}-+oooooooooo+-  ${c5}:+oooooo/.
${c4}-oooooooooo+:  ${c3}-+oooooo+-  ${c5}:+oooooooooo-
${c4}.+ooooooooooo+:  ${c3}-+oo+-  ${c5}:+ooooooooooo+.
${c4}  -+ooooooooooo+:  ${c3}..  ${c5}:+ooooooooooo+-
${c4}    -+ooooooooooo+:  ${c5}:+ooooooooooo+-
${c4}      -+oooooooooo+:${c5}:+oooooooooo+-
${c4}        -+oooooo+:    ${c5}:+oooooo+-
${c4}          -+oo+:        ${c5}:+oo+-
${c4}            ..            ${c5}..
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

alpine_small = AsciiArt(match=r'''"alpine_small"''', color='4 7', ascii=r"""
${c1}   /\ /\
  /${c2}/ ${c1}\  \
 /${c2}/   ${c1}\  \
/${c2}//    ${c1}\  \
${c2}//      ${c1}\  \
         \
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

slackware_small = AsciiArt(match=r'''"slackware_small"*''', color='4 7 1', ascii=r"""
${c1}   ________
  /  ______|
  | |______
  \______  \
   ______| |
| |________/
|____________
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

biglinux = AsciiArt(match=r'''"BigLinux"*''', color='6 11 4', ascii=r"""
${c1}                                 ...
                              :OWMMMNd.
                            :NMMMMMMMMWc
                  okkl.    kMMMMMW0xdOWMl
  :             xMMMMMW.  kMMMMNc      lW.
 :x             NMMMMMO  ,MMMM0.        'l
 Xx              "lkk"   kMMMX      .okx,
${c2}.MX      .cc;.    .xXKx. KMMM:    .OMMMMMl
:MM'   'KMMMMWK:  0MMMMk xMMM.   lWMMMMMMM'
cMMN:;xMMMMk::MMO oMMMMX .XMM. .KMMMWOOMMMd
'MMMMMMMMN,   NMMx OMMMMl .kM0OMMMMk.  ;MMd
 xMMMMMMd    .MMMW  :NMMMd  .ckKKx'     KMc
  dWMNd.     oMMMN    lkNMX,            oM.
 ;.         ;MMMMx      "MM:.           cO
${c3} .X.       oMMMMW.                      l.
  dMk:..;xWMMMMW,
   kMMMMMMMMMMX.
    :XMMMMMMK:
      ':MM:"      Made in Brazil
""")
    


import os
import platform
import subprocess
import sys
from pathlib import Path

from .color_util import printc
from .py import run_py


def run_rust():
    # Find the rust executable
    pd = Path(__file__).parent.joinpath('rust')
    pd = pd.joinpath('hyfetch.exe' if platform.system() == 'Windows' else 'hyfetch')
    if not pd.exists():
        if 'HYFETCH_DONT_WARN_RUST' not in os.environ:
            printc('&cThe executable for hyfetch v2 (rust) is not found, falling back to legacy v1.99.∞ (python).\n'
                   'You can add environment variable HYFETCH_DONT_WARN_RUST=1 to suppress this warning.\n')
        run_py()
        return

    # Run the rust executable, passing in all arguments
    subprocess.run([str(pd)] + sys.argv[1:])


if __name__ == '__main__':
    run_rust()



# This file is automatically generated. Please do not modify.

from . import AsciiArt

ubuntu_mate = AsciiArt(match=r'''"Ubuntu MATE"* | "Ubuntu-MATE"*''', color='2 7', ascii=r"""
${c1}            .:/+oossssoo+/:.`
        `:+ssssssssssssssssss+:`
      -+sssssssssssssss${c2}y${c1}ssssssss+-
    .osssssssssssss${c2}yy${c1}ss${c2}mMmh${c1}ssssssso.
   /sssssssss${c2}ydmNNNmmd${c1}s${c2}mMMMMNdy${c1}sssss/
 `+ssssssss${c2}hNNdy${c1}sssssss${c2}mMMMMNdy${c1}ssssss+`
 +sssssss${c2}yNNh${c1}ss${c2}hmNNNNm${c1}s${c2}mMmh${c1}s${c2}ydy${c1}sssssss+
-sssss${c2}y${c1}ss${c2}Nm${c1}ss${c2}hNNh${c1}ssssss${c2}y${c1}s${c2}hh${c1}ss${c2}mMy${c1}sssssss-
+ssss${c2}yMNdy${c1}ss${c2}hMd${c1}ssssssssss${c2}hMd${c1}ss${c2}NN${c1}sssssss+
sssss${c2}yMMMMMmh${c1}sssssssssssss${c2}NM${c1}ss${c2}dMy${c1}sssssss
sssss${c2}yMMMMMmhy${c1}ssssssssssss${c2}NM${c1}ss${c2}dMy${c1}sssssss
+ssss${c2}yMNdy${c1}ss${c2}hMd${c1}ssssssssss${c2}hMd${c1}ss${c2}NN${c1}sssssss+
-sssss${c2}y${c1}ss${c2}Nm${c1}ss${c2}hNNh${c1}ssssssss${c2}dh${c1}ss${c2}mMy${c1}sssssss-
 +sssssss${c2}yNNh${c1}ss${c2}hmNNNNm${c1}s${c2}mNmh${c1}s${c2}ymy${c1}sssssss+
  +ssssssss${c2}hNNdy${c1}sssssss${c2}mMMMMmhy${c1}ssssss+
   /sssssssss${c2}ydmNNNNmd${c1}s${c2}mMMMMNdh${c1}sssss/
    .osssssssssssss${c2}yy${c1}ss${c2}mMmdy${c1}sssssso.
      -+sssssssssssssss${c2}y${c1}ssssssss+-
        `:+ssssssssssssssssss+:`
            .:/+oossssoo+/:.
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

arch_old = AsciiArt(match=r'''"arch_old"''', color='6 7 1', ascii=r"""
${c1}             __
         _=(SDGJT=_
       _GTDJHGGFCVS)
      ,GTDJGGDTDFBGX0
${c1}     JDJDIJHRORVFSBSVL${c2}-=+=,_
${c1}    IJFDUFHJNXIXCDXDSV,${c2}  "DEBL
${c1}   [LKDSDJTDU=OUSCSBFLD.${c2}   '?ZWX,
${c1}  ,LMDSDSWH'     `DCBOSI${c2}     DRDS],
${c1}  SDDFDFH'         !YEWD,${c2}   )HDROD
${c1} !KMDOCG            &GSU|${c2}\_GFHRGO\'
${c1} HKLSGP'${c2}           __${c1}\TKM0${c2}\GHRBV)'
${c1}JSNRVW'${c2}       __+MNAEC${c1}\IOI,${c2}\BN'
${c1}HELK['${c2}    __,=OFFXCBGHC${c1}\FD)
${c1}?KGHE ${c2}\_-#DASDFLSV='${c1}    'EF
'EHTI                    !H
 `0F'                    '!
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

redhat = AsciiArt(match=r'''"Redhat"* | "Red Hat"* | "rhel"*''', color='1', ascii=r"""
${c1}           .MMM..:MMMMMMM
          MMMMMMMMMMMMMMMMMM
          MMMMMMMMMMMMMMMMMMMM.
         MMMMMMMMMMMMMMMMMMMMMM
        ,MMMMMMMMMMMMMMMMMMMMMM:
        MMMMMMMMMMMMMMMMMMMMMMMM
  .MMMM'  MMMMMMMMMMMMMMMMMMMMMM
 MMMMMM    `MMMMMMMMMMMMMMMMMMMM.
MMMMMMMM      MMMMMMMMMMMMMMMMMM .
MMMMMMMMM.       `MMMMMMMMMMMMM' MM.
MMMMMMMMMMM.                     MMMM
`MMMMMMMMMMMMM.                 ,MMMMM.
 `MMMMMMMMMMMMMMMMM.          ,MMMMMMMM.
    MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
      MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM:
         MMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
            `MMMMMMMMMMMMMMMMMMMMMMMM:
                ``MMMMMMMMMMMMMMMMM'
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

sailfish = AsciiArt(match=r'''"Sailfish"*''', color='4 5 7 6', ascii=r"""
${c1}                 _a@b
              _#b (b
            _@@   @_         _,
          _#^@ _#*^^*gg,aa@^^
          #- @@^  _a@^^
          @_  *g#b
          ^@_   ^@_
            ^@_   @
             @(b (b
            #b(b#^
          _@_#@^
       _a@a*^
   ,a@*^
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

openeuler = AsciiArt(match=r'''"openEuler"*''', color='4 7 1', ascii=r"""
${c1}                 `.cc.`
             ``.cccccccc..`
          `.cccccccccccccccc.`
      ``.cccccccccccccccccccccc.``
   `..cccccccccccccccccccccccccccc..`
`.ccccccccccccccc${c2}/++/${c1}ccccccccccccccccc.`
.ccccccccccccccc${c2}mNMMNdo+oso+${c1}ccccccccccc.
.cccccccccc${c2}/++odms+//+mMMMMm/:+syso/${c1}cccc
.ccccccccc${c2}yNNMMMs:::/::+o+/:${c1}c${c2}dMMMMMm${c1}cccc
.ccccccc${c2}:+NmdyyhNNmNNNd:${c1}ccccc${c1}${c2}:oyyyo:${c1}cccc
.ccc${c2}:ohdmMs:${c1}cccc${c2}+mNMNmy${c1}ccccccccccccccccc
.cc${c2}/NMMMMMo////:${c1}c${c2}:///:${c1}cccccccccccccccccc
.cc${c2}:syysyNMNNNMNy${c1}ccccccccccccccccccccccc
.cccccccc${c2}+MMMMMNy${c1}c${c2}:/+++/${c1}cccccccccccccccc
.ccccccccc${c2}ohhhs/${c1}c${c2}omMMMMNh${c1}ccccccccccccccc
.ccccccccccccccc${c2}:MMMMMMMM/${c1}cccccccccccccc
.cccccccccccccccc${c2}sNNNNNd+${c1}cccccccccccccc.
`..cccccccccccccccc${c2}/+/:${c1}cccccccccccccc..`
   ``.cccccccccccccccccccccccccccc.``
       `.cccccccccccccccccccccc.`
          ``.cccccccccccccc.``
              `.cccccccc.`
                 `....`
""")
    


try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

AnsiMode = Literal['default', 'ansi', '8bit', 'rgb']
LightDark = Literal['light', 'dark']
BackendLiteral = Literal["neofetch", "fastfetch"]
ColorAlignMode = Literal['horizontal', 'vertical', 'custom']
ColorSpacing = Literal['equal', 'weighted']



# This file is automatically generated. Please do not modify.

from . import AsciiArt

cleanjaro_small = AsciiArt(match=r'''"cleanjaro_small"*''', color='7 7', ascii=r"""
${c1}█████ ██████████
█████ ██████████
█████
█████
█████
████████████████
████████████████
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

pclinuxos = AsciiArt(match=r'''"PCLinuxOS"*''', color='4 7 1', ascii=r"""
${c1}            mhhhyyyyhhhdN
        dyssyhhhhhhhhhhhssyhN
     Nysyhhyo/:-.....-/oyhhhssd
   Nsshhy+.              `/shhysm
  dohhy/                    -shhsy
 dohhs`                       /hhys
N+hho   ${c2}+ssssss+-   .+syhys+   ${c1}/hhsy
ohhh`   ${c2}ymmo++hmm+`smmy/::+y`   ${c1}shh+
+hho    ${c2}ymm-  /mmy+mms          ${c1}:hhod
/hh+    ${c2}ymmhhdmmh.smm/          ${c1}.hhsh
+hhs    ${c2}ymm+::-`  /mmy`    `    ${c1}/hh+m
yyhh-   ${c2}ymm-       /dmdyosyd`  ${c1}`yhh+
 ohhy`  ${c2}://`         -/+++/-   ${c1}ohhom
 N+hhy-                      `shhoh
   sshho.                  `+hhyom
    dsyhhs/.            `:ohhhoy
      dysyhhhso///://+syhhhssh
         dhyssyhhhhhhyssyyhN
              mddhdhdmN
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

container_linux_by_coreos = AsciiArt(match=r'''"Container Linux by CoreOS"* | "Container_Linux"* | "Fedora CoreOS" | "CoreOS"*''', color='4 7 1', ascii=r"""
${c1}                .....
          .';:cccccccc:;'.
        ':ccccclc${c3}lllllllll${c1}cc:.
     .;cccccccc${c3}lllllllllllllll${c1}c,
    ;clllccccc${c3}llllllllllllllllll${c1}c,
  .cllclccccc${c3}lllll${c2}lll${c3}llllllllllll${c1}c:
  ccclclcccc${c3}cllll${c2}kWMMNKk${c3}llllllllll${c1}c:
 :ccclclcccc${c3}llll${c2}oWMMMMMMWO${c3}lllllllll${c1}c,
.ccllllllccc${c3}clll${c2}OMMMMMMMMM0${c3}lllllllll${c1}c
.lllllclcccc${c3}llll${c2}KMMMMMMMMMMo${c3}llllllll${c1}c.
.lllllllcccc${c3}clll${c2}KMMMMMMMMN0${c3}lllllllll${c1}c.
.cclllllcccc${c3}lllld${c2}xkkxxdo${c3}llllllllllc${c1}lc
 :cccllllllcccc${c3}lllccllllcclccc${c1}cccccc;
 .ccclllllllcccccccc${c3}lll${c1}ccccclccccccc
  .cllllllllllclcccclccclccllllcllc
    :cllllllllccclcllllllllllllcc;
     .cccccccccccccclcccccccccc:.
       .;cccclccccccllllllccc,.
          .';ccccclllccc:;..
                .....
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

nurunner = AsciiArt(match=r'''"Nurunner"*''', color='4', ascii=r"""
${c1}                  ,xc
                ;00cxXl
              ;K0,   .xNo.
            :KO'       .lXx.
          cXk.    ;xl     cXk.
        cXk.    ;k:.,xo.    cXk.
     .lXx.    :x::0MNl,dd.    :KO,
   .xNx.    cx;:KMMMMMNo'dx.    ;KK;
 .dNl.    cd,cXMMMMMMMMMWd,ox'    'OK:
;WK.    'K,.KMMMMMMMMMMMMMWc.Kx     lMO
 'OK:    'dl'xWMMMMMMMMMM0::x:    'OK:
   .kNo    .xo'xWMMMMMM0;:O:    ;KK;
     .dXd.   .do,oNMMO;ck:    ;00,
        oNd.   .dx,;'cO;    ;K0,
          oNx.    okk;    ;K0,
            lXx.        :KO'
              cKk'    cXk.
                ;00:lXx.
                  ,kd.
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

openbsd_small = AsciiArt(match=r'''"openbsd_small"''', color='3 7 6 1 8', ascii=r"""
${c1}      _____
    \-     -/
 \_/         \
 |        ${c2}O O${c1} |
 |_  <   )  3 )
 / \         /
    /-_____-\
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

parrot = AsciiArt(match=r'''"Parrot"*''', color='6 7', ascii=r"""
${c1}  `:oho/-`
`mMMMMMMMMMMMNmmdhy-
 dMMMMMMMMMMMMMMMMMMs`
 +MMsohNMMMMMMMMMMMMMm/
 .My   .+dMMMMMMMMMMMMMh.
  +       :NMMMMMMMMMMMMNo
           `yMMMMMMMMMMMMMm:
             /NMMMMMMMMMMMMMy`
              .hMMMMMMMMMMMMMN+
                  ``-NMMMMMMMMMd-
                     /MMMMMMMMMMMs`
                      mMMMMMMMsyNMN/
                      +MMMMMMMo  :sNh.
                      `NMMMMMMm     -o/
                       oMMMMMMM.
                       `NMMMMMM+
                        +MMd/NMh
                         mMm -mN`
                         /MM  `h:
                          dM`   .
                          :M-
                           d:
                           -+
                            -
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

nixos_small = AsciiArt(match=r'''"nixos_small"''', color='4 6', ascii=r"""
${c1}  \\  \\ //
 ==\\__\\/ //
   //   \\//
==//     //==
 //\\___//
// /\\  \\==
  // \\  \\
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

instantos = AsciiArt(match=r'''"instantOS"*''', color='4 6', ascii=r"""

${c1}
     'cx0XWWMMWNKOd:'.
  .;kNMMMMMMMMMMMMMWNKd'
 'kNMMMMMMWNNNWMMMMMMMMXo.
,0MMMMMW0o;'..,:dKWMMMMMWx.
OMMMMMXl.        .xNMMMMMNo
WMMMMNl           .kWWMMMMO'
MMMMMX;            oNWMMMMK,
NMMMMWo           .OWMMMMMK,
kWMMMMNd.        ,kWMMMMMMK,
'kWMMMMWXxl:;;:okNMMMMMMMMK,
 .oXMMMMMMMWWWMMMMMMMMMMMMK,
   'oKWMMMMMMMMMMMMMMMMMMMK,
     .;lxOKXXXXXXXXXXXXXXXO;......
          ................,d0000000kd:.
                          .kMMMMMMMMMW0;
                          .kMMMMMMMMMMMX
                          .xMMMMMMMMMMMW
                           cXMMMMMMMMMM0
                            :0WMMMMMMNx,
                             .o0NMWNOc.
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

live_raizo = AsciiArt(match=r'''"Live Raizo"* | "Live_Raizo"*''', color='3', ascii=r"""
${c1}             `......`
        -+shmNMMMMMMNmhs/.
     :smMMMMMmmhyyhmmMMMMMmo-
   -hMMMMd+:. `----` .:odMMMMh-
 `hMMMN+. .odNMMMMMMNdo. .yMMMMs`
 hMMMd. -dMMMMmdhhdNMMMNh` .mMMMh
oMMMm` :MMMNs.:sddy:-sMMMN- `NMMM+
mMMMs  dMMMo sMMMMMMd yMMMd  sMMMm
----`  .---` oNMMMMMh `---.  .----
              .sMMy:
               /MM/
              +dMMms.
             hMMMMMMN
            `dMMMMMMm:
      .+ss+sMNysMMoomMd+ss+.
     +MMMMMMN` +MM/  hMMMMMNs
     sMMMMMMm-hNMMMd-hMMMMMMd
      :yddh+`hMMMMMMN :yddy/`
             .hMMMMd:
               `..`
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

chimera = AsciiArt(match=r'''"Chimera"*''', color='1 5 4 1', ascii=r"""
${c3}ddddddddddddddc  ${c1},cc:
${c3}ddddddddddddddc  ${c1},cc:
${c3}ddddddddddddddd  ${c1},cc:
${c3}ddddddddddddl:'  ${c1},cc:
${c3}dddddddddl'    ${c1}..;cc:
${c3}dddddddo.   ${c1},:cccccc:
${c3}ddddddl   ${c1},ccc:'''''
${c3}dddddo.  ${c1};ccc.          ............
        .ccc.           cccccccccccc
${c2}......  ${c1}.ccc.          .ccc'''''''''
${c2}OOOOOk.  ${c1};ccc.        .ccc;   ......
${c2}OOOOOOd   ${c1}'ccc:,....,:ccc'   ${c4}coooooo
${c2}OOOOOOOx.   ${c1}':cccccccc:'   ${c4}.looooooo
${c2}OOOOOOOOOd,     ${c1}`'''`     ${c4}.coooooooo
${c2}OOOOOOOOOOOOdc,.    ${c4}..,coooooooooooo
${c2}OOOOOOOOOOOOOOOO'  ${c4}.oooooooooooooooo
${c2}OOOOOOOOOOOOOOOO'  ${c4}.oooooooooooooooo
${c2}OOOOOOOOOOOOOOOO'  ${c4}.oooooooooooooooo
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

pacbsd = AsciiArt(match=r'''"PacBSD"*''', color='1 7 3', ascii=r"""
${c1}      :+sMs.
  `:ddNMd-                         -o--`
 -sMMMMh:                          `+N+``
 yMMMMMs`     .....-/-...           `mNh/
 yMMMMMmh+-`:sdmmmmmmMmmmmddy+-``./ddNMMm
 yNMMNMMMMNdyyNNMMMMMMMMMMMMMMMhyshNmMMMm
 :yMMMMMMMMMNdooNMMMMMMMMMMMMMMMMNmy:mMMd
  +MMMMMMMMMmy:sNMMMMMMMMMMMMMMMMMMMmshs-
  :hNMMMMMMN+-+MMMMMMMMMMMMMMMMMMMMMMMs.
 .omysmNNhy/+yNMMMMMMMMMMNMMMMMMMMMNdNNy-
 /hMM:::::/hNMMMMMMMMMMMm/-yNMMMMMMN.mMNh`
.hMMMMdhdMMMMMMMMMMMMMMmo  `sMMMMMMN mMMm-
:dMMMMMMMMMMMMMMMMMMMMMdo+  oMMMMMMN`smMNo`
/dMMMMMMMMMMMMMMMMMMMMMNd/` :yMMMMMN:-hMMM.
:dMMMMMMMMMMMMMMMMMMMMMNh`  oMMMMMMNo/dMNN`
:hMMMMMMMMMMMMMMMMMMMMMMNs--sMMMMMMMNNmy++`
 sNMMMMMMMMMMMMMMMMMMMMMMMmmNMMMMMMNho::o.
 :yMMMMMMMMMMMMMNho+sydNNNNNNNmysso/` -//
  /dMMMMMMMMMMMMMs-  ````````..``
   .oMMMMMMMMMMMMNs`               ./y:`
     +dNMMNMMMMMMMmy`          ``./ys.
      `/hMMMMMMMMMMMNo-``    `.+yy+-`
        `-/hmNMNMMMMMMmmddddhhy/-`
            `-+oooyMMMdsoo+/:.
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

furreto = AsciiArt(match=r'''"Furreto"*''', color='211 255 225 199', ascii=r"""
${c1}
               ${c1}.xOOko      .odd,
              o${c3}X${c2}WWK${c3}O${c1}OOO. 'O${c3}N${c2}WW${c3}0${c1}kkk.
             .k0${c3}XKO${c1}OOOOOcOO${c2}NWN${c1}OOOOO.
              xOOOOOOOOOkkOOOOOOOOO;
       .O0OkkocxO000000kcdk0OO0OOkx
       k${c3}W${c2}MX${c1}kkkkloxkkkx;  :dxxxxddc...
      'kO0OOOOOkc          .cl:..kk0KK0Okc
      ;kOOO0000xd.  dO00000Oo  .xkO${c3}N${c2}MM${c3}X${c1}OOOO
    .dddxkOOOkddc.k${c3}K${c2}NWW${c3}N${c1}000000l.ddk${c2}00${c1}00OOOO.
     'dd:;ddddd;.d${c3}K${c2}MMM${c3}W${c1}K00KKK0O::ddxkO00Oko
             .okxkOKK0kkOO00KKOxxlodddddddl
           .00OOkkkkkkkkOOO00OOOO0O;  .dddl
          'kkkkkxxkkkkkkkOOkxdxkxxddd.
          cddddddddxxkkkkk${c4}xddddddddddo
          'ddddddodddddddddddddddddddc
           .ddddddodddddddddodddddddc
                    .odddo.

                   .kOOkkk;
                  lkKWNkkkxc
                 kkxkkkkkkx.
                  ,,..xxx.
""")
    


"""
This version of color_scale is a special version made without numpy dependency. The numpy version
would be faster, but numpy is 11 MB large. In comparison, hyfetch 1.0.7 is only 105 kB, so it's not
a good idea to depend on numpy.
"""
from __future__ import annotations

from .color_util import RGB


def create_gradient_hex(colors: list[str], resolution: int = 300) -> list[RGB]:
    """
    Create gradient array from hex
    """
    colors = [RGB.from_hex(s) for s in colors]
    return create_gradient(colors, resolution)


def create_gradient(colors: list[RGB], resolution: int) -> list[RGB]:
    """
    Create gradient array

    Usage: arr[ratio / len(arr)] = Scaled gradient color at that point
    """
    result = []

    # Create gradient mapping
    for i in range(len(colors) - 1):
        c1 = colors[i]
        c2 = colors[i + 1]
        bi = i * resolution

        for ri in range(resolution):
            ratio = ri / resolution
            r = int(c2.r * ratio + c1.r * (1 - ratio))
            g = int(c2.g * ratio + c1.g * (1 - ratio))
            b = int(c2.b * ratio + c1.b * (1 - ratio))
            result.append(RGB(r, g, b))

    return result


def get_raw(gradient: list[RGB], ratio: float) -> RGB:
    """
    :param gradient: Gradient array (2d)
    :param ratio: Between 0-1
    :return: RGB subarray (1d, has 3 values)
    """
    if ratio == 1:
        return gradient[-1]

    i = int(ratio * len(gradient))
    return gradient[i]


class Scale:
    colors: list[RGB]
    rgb: list[RGB]

    def __init__(self, scale: list[str], resolution: int = 300):
        self.colors = [RGB.from_hex(s) for s in scale]
        self.rgb = create_gradient(self.colors, resolution)

    def __call__(self, ratio: float) -> RGB:
        """
        :param ratio: Between 0-1
        """
        return get_raw(self.rgb, ratio)


def test_color_scale():
    scale = Scale(['#232323', '#4F1879', '#B43A78', '#F98766', '#FCFAC0'])

    colors = 100
    for i in range(colors + 1):
        print(scale(i / colors).to_ansi_rgb(False), end=' ')



# This file is automatically generated. Please do not modify.

from . import AsciiArt

ultramarine_linux = AsciiArt(match=r'''"Ultramarine Linux"* | "ultramarine"*''', color='4 7', ascii=r"""
${c1}            .cd0NNNNNNNXOdc.
        .:xKNNNNNNNNNNNNNNNNKd;.
      ,dXNNNNNNNNNNNNNNNNNNNNNNNd,
    'ONNNNNNNNNNNNNNNNNNNNNNNNNNNNO'
  .xNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNk.
 .0NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN0.
.0NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN0.
dNNNNNNNNNNNNWWWWWWWWNNNNNNNNNNNNNNNNNNd
NNNNNNNNNNNNNW${c2}MMMMMMMM${c1}WWNNNNNNNNNNNNNNNN
NNNNNNNNNNNNNNW${c2}MMMMMMMMM${c1}WWNNNNNNNNNNNNNN
NNNNNNNNNNNNNNW${c2}MMMMMMMMMMMM${c1}WNNNNNNNNNNNN
NNNNNNNNNNWWW${c2}MMMMMMMMMMMMMMMM${c1}WWWNNNNNNNX
oNWWWW${c2}MMMMMMMMMMMMMMMMMMMMMMMMMMMM${c1}WWWNNo
 OW${c2}MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM${c1}WO
 .OW${c2}MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM${c1}WO.
   lNW${c2}MMMMMMMMMMMMMMMMMMMMMMMMMMMM${c1}WNl
    .dNW${c2}MMMMMMMMMMMMMMMMMMMMMMMM${c1}WNd.
      .cKW${c2}MMMMMMMMMMMMMMMMMMMM${c1}WKc.
         'oOXWWW${c2}MMMMMMMM${c1}WWWXOl.
             ;lkXNNNNNNXkl'
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

langitketujuh_old = AsciiArt(match=r'''"LangitKetujuh_old"''', color='7 4', ascii=r"""
${c1}
   L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L
      'L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L
   L7L.   'L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L
   L7L7L7L                             L7L7L7L
   L7L7L7L                             L7L7L7L
   L7L7L7L             L7L7L7L7L7L7L7L7L7L7L7L
   L7L7L7L                'L7L7L7L7L7L7L7L7L7L
   L7L7L7L                    'L7L7L7L7L7L7L7L
   L7L7L7L                             L7L7L7L
   L7L7L7L                             L7L7L7L
   L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L.   'L7L
   L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L.
   L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L7L
${c2}
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

pentoo = AsciiArt(match=r'''"Pentoo"*''', color='5 7', ascii=r"""
${c2}           `:oydNNMMMMNNdyo:`
        :yNMMMMMMMMMMMMMMMMNy:
      :dMMMMMMMMMMMMMMMMMMMMMMd:
     oMMMMMMMho/-....-/ohMMMMMMMo
    oMMMMMMy.            .yMMMMMMo
   .MMMMMMo                oMMMMMM.
   +MMMMMm                  mMMMMM+
   oMMMMMh                  hMMMMMo
 //hMMMMMm//${c1}`${c2}          ${c1}`${c2}////mMMMMMh//
MMMMMMMMMMM${c1}/${c2}      ${c1}/o/`${c2}  ${c1}.${c2}smMMMMMMMMMMM
MMMMMMMMMMm      ${c1}`NMN:${c2}    ${c1}.${c2}yMMMMMMMMMM
MMMMMMMMMMMh${c1}:.${c2}              dMMMMMMMMM
MMMMMMMMMMMMMy${c1}.${c2}            ${c1}-${c2}NMMMMMMMMM
MMMMMMMMMMMd:${c1}`${c2}           ${c1}-${c2}yNMMMMMMMMMM
MMMMMMMMMMh${c1}`${c2}          ${c1}./${c2}hNMMMMMMMMMMMM
MMMMMMMMMM${c1}s${c2}        ${c1}.:${c2}ymMMMMMMMMMMMMMMM
MMMMMMMMMMN${c1}s:..-/${c2}ohNMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
 MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

elementary = AsciiArt(match=r'''"Elementary"*''', color='4 7 1', ascii=r"""
${c2}         eeeeeeeeeeeeeeeee
      eeeeeeeeeeeeeeeeeeeeeee
    eeeee  eeeeeeeeeeee   eeeee
  eeee   eeeee       eee     eeee
 eeee   eeee          eee     eeee
eee    eee            eee       eee
eee   eee            eee        eee
ee    eee           eeee       eeee
ee    eee         eeeee      eeeeee
ee    eee       eeeee      eeeee ee
eee   eeee   eeeeee      eeeee  eee
eee    eeeeeeeeee     eeeeee    eee
 eeeeeeeeeeeeeeeeeeeeeeee    eeeee
  eeeeeeee eeeeeeeeeeee      eeee
    eeeee                 eeeee
      eeeeeee         eeeeeee
         eeeeeeeeeeeeeeeee
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

opensuse_small = AsciiArt(match=r'''"opensuse_small" | "suse_small"*''', color='2 7', ascii=r"""
${c1}  _______
__|   __ \
     / .\ \
     \__/ |
   _______|
   \_______
__________/
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

amazon = AsciiArt(match=r'''"Amazon"*''', color='3 7', ascii=r"""
${c1}             `-/oydNNdyo:.`
      `.:+shmMMMMMMMMMMMMMMmhs+:.`
    -+hNNMMMMMMMMMMMMMMMMMMMMMMNNho-
.``      -/+shmNNMMMMMMNNmhs+/-      ``.
dNmhs+:.       `.:/oo/:.`       .:+shmNd
dMMMMMMMNdhs+:..        ..:+shdNMMMMMMMd
dMMMMMMMMMMMMMMNds    odNMMMMMMMMMMMMMMd
dMMMMMMMMMMMMMMMMh    yMMMMMMMMMMMMMMMMd
dMMMMMMMMMMMMMMMMh    yMMMMMMMMMMMMMMMMd
dMMMMMMMMMMMMMMMMh    yMMMMMMMMMMMMMMMMd
dMMMMMMMMMMMMMMMMh    yMMMMMMMMMMMMMMMMd
dMMMMMMMMMMMMMMMMh    yMMMMMMMMMMMMMMMMd
dMMMMMMMMMMMMMMMMh    yMMMMMMMMMMMMMMMMd
dMMMMMMMMMMMMMMMMh    yMMMMMMMMMMMMMMMMd
dMMMMMMMMMMMMMMMMh    yMMMMMMMMMMMMMMMMd
dMMMMMMMMMMMMMMMMh    yMMMMMMMMMMMMMMMMd
.:+ydNMMMMMMMMMMMh    yMMMMMMMMMMMNdy+:.
     `.:+shNMMMMMh    yMMMMMNhs+:``
            `-+shy    shs+:`
""")
    


#!/usr/bin/env python3
"""
This script turns readme shorthand pull request references (i.e. dylanaraps/neofetch#1946) into full
GitHub pull request links.
"""
import regex as re
from pathlib import Path


RE_SHORTHAND = re.compile(r"(\w+?)/(\w+?)#(\d+)")
MY_RE = re.compile(r"(?<=[^\w\[])#(\d+)")


def reformat_readme():
    readme = Path('README.md').read_text()

    readme = re.sub(RE_SHORTHAND, r'[\1#\3](https://github.com/\1/\2/pull/\3)', readme)
    readme = re.sub(MY_RE, r'[#\1](https://github.com/hykilpikonna/hyfetch/pull/\1)', readme)

    Path('README.md').write_text(readme)


if __name__ == '__main__':
    reformat_readme()



from __future__ import annotations

import colorsys
from dataclasses import dataclass, astuple

from .constants import GLOBAL_CFG
from .types import *

MINECRAFT_COLORS = [

    # Minecraft formatting codes
    # ==========================
    "&0/\033[38;5;0m", "&1/\033[38;5;4m",  "&2/\033[38;5;2m",  "&3/\033[38;5;6m",
    "&4/\033[38;5;1m", "&5/\033[38;5;5m",  "&6/\033[38;5;3m",  "&7/\033[38;5;7m",
    "&8/\033[38;5;8m", "&9/\033[38;5;12m", "&a/\033[38;5;10m", "&b/\033[38;5;14m",
    "&c/\033[38;5;9m", "&d/\033[38;5;13m", "&e/\033[38;5;11m", "&f/\033[38;5;15m",
    "&l/\033[1m",   # Enable bold text
    "&o/\033[3m",   # Enable italic text
    "&n/\033[4m",   # Enable underlined text
    "&k/\033[8m",   # Enable hidden text
    "&m/\033[9m",   # Enable strikethrough text
    "&r/\033[0m",   # Reset everything

    # Extended codes (not officially in Minecraft)
    # ============================================
    "&-/\n",        # Line break
    "&~/\033[39m",  # Reset text color
    "&*/\033[49m",  # Reset background color
    "&L/\033[22m",  # Disable bold text
    "&O/\033[23m",  # Disable italic text
    "&N/\033[24m",  # Disable underlined text
    "&K/\033[28m",  # Disable hidden text
    "&M/\033[29m",  # Disable strikethrough text

]
MINECRAFT_COLORS = [(r[:2], r[3:]) for r in MINECRAFT_COLORS]


def color(msg: str) -> str:
    """
    Replace extended minecraft color codes in string
    :param msg: Message with minecraft color codes
    :return: Message with escape codes
    """
    for code, esc in MINECRAFT_COLORS:
        msg = msg.replace(code, esc)

    while '&gf(' in msg or '&gb(' in msg:
        i = msg.index('&gf(') if '&gf(' in msg else msg.index('&gb(')
        end = msg.index(')', i)
        code = msg[i + 4:end]
        fore = msg[i + 2] == 'f'

        if code.startswith('#'):
            rgb = tuple(int(code.lstrip('#')[i:i+2], 16) for i in (0, 2, 4))
        else:
            code = code.replace(',', ' ').replace(';', ' ').replace('  ', ' ')
            rgb = tuple(int(c) for c in code.split(' '))

        msg = msg[:i] + RGB(*rgb).to_ansi(foreground=fore) + msg[end + 1:]

    return msg


def printc(msg: str):
    """
    Print with color
    :param msg: Message with minecraft color codes
    """
    print(color(msg + '&r'))


def clear_screen(title: str = ''):
    """
    Clear screen using ANSI escape codes
    """
    if not GLOBAL_CFG.debug:
        print('\033[2J\033[H', end='')

    if title:
        print()
        printc(title)
        print()


def redistribute_rgb(r: int, g: int, b: int) -> tuple[int, int, int]:
    """
    Redistribute RGB after lightening

    Credit: https://stackoverflow.com/a/141943/7346633
    """
    threshold = 255.999
    m = max(r, g, b)
    if m <= threshold:
        return int(r), int(g), int(b)
    total = r + g + b
    if total >= 3 * threshold:
        return int(threshold), int(threshold), int(threshold)
    x = (3 * threshold - total) / (3 * m - total)
    gray = threshold - x * m
    return int(gray + x * r), int(gray + x * g), int(gray + x * b)


@dataclass(unsafe_hash=True)
class HSL:
    h: float
    s: float
    l: float

    def __iter__(self):
        return iter(astuple(self))

    def rgb(self) -> RGB:
        return RGB(*[round(v * 255.0) for v in colorsys.hls_to_rgb(self.h, self.l, self.s)])


@dataclass(unsafe_hash=True)
class RGB:
    r: int
    g: int
    b: int

    def __iter__(self):
        return iter(astuple(self))

    @classmethod
    def from_hex(cls, hex: str) -> "RGB":
        """
        Create color from hex code

        >>> RGB.from_hex('#FFAAB7')
        RGB(r=255, g=170, b=183)

        :param hex: Hex color code
        :return: RGB object
        """
        hex = hex.lstrip("#")
        r = int(hex[0:2], 16)
        g = int(hex[2:4], 16)
        b = int(hex[4:6], 16)
        return cls(r, g, b)

    def to_ansi_rgb(self, foreground: bool = True) -> str:
        """
        Convert RGB to ANSI TrueColor (RGB) Escape Code.

        This uses the 24-bit color encoding (an uint8 for each color value), and supports 16 million
        colors. However, not all terminal emulators support this escape code. (For example, IntelliJ
        debug console doesn't support it).

        Currently, we do not know how to detect whether a terminal environment supports ANSI RGB. If
        you have any thoughts, feel free to submit an issue on our Github page!

        :param foreground: Whether the color is for foreground text or background color
        :return: ANSI RGB escape code like \033[38;2;255;100;0m
        """
        c = '38' if foreground else '48'
        return f'\033[{c};2;{self.r};{self.g};{self.b}m'

    def to_ansi_8bit(self, foreground: bool = True) -> str:
        """
        Convert RGB to ANSI 8bit 256 Color Escape Code.

        This encoding supports 256 colors in total.

        :return: ANSI 256 escape code like \033[38;5;206m'
        """
        r, g, b = self.r, self.g, self.b
        sep = 42.5

        while True:
            if r < sep or g < sep or b < sep:
                gray = r < sep and g < sep and b < sep
                break
            sep += 42.5

        if gray:
            color = 232 + (r + g + b) / 33
        else:
            color = 16 + int(r / 256. * 6) * 36 + int(g / 256. * 6) * 6 + int(b / 256. * 6)

        c = '38' if foreground else '48'
        return f'\033[{c};5;{int(color)}m'

    def to_ansi_16(self, foreground: bool = True) -> str:
        """
        Convert RGB to ANSI 16 Color Escape Code

        :return: ANSI 16 escape code
        """
        raise NotImplementedError()

    def to_ansi(self, mode: AnsiMode | None = None, foreground: bool = True):
        if not mode:
            mode = GLOBAL_CFG.color_mode
        if mode == 'rgb':
            return self.to_ansi_rgb(foreground)
        if mode == '8bit':
            return self.to_ansi_8bit(foreground)
        if mode == 'ansi':
            return self.to_ansi_16(foreground)

    def lighten(self, multiplier: float) -> 'RGB':
        """
        Lighten the color by a multiplier

        :param multiplier: Multiplier
        :return: Lightened color (original isn't modified)
        """
        return RGB(*redistribute_rgb(*[v * multiplier for v in self]))

    def hsl(self) -> HSL:
        h, l, s = colorsys.rgb_to_hls(*[v / 255.0 for v in self])
        return HSL(h, s, l)

    def set_light(self, light: float, at_least: bool | None = None, at_most: bool | None = None) -> 'RGB':
        """
        Set HSL lightness value

        :param light: Lightness value (0-1)
        :param at_least: Set the lightness to at least this value (no change if greater)
        :param at_most: Set the lightness to at most this value (no change if lesser)
        :return: New color (original isn't modified)
        """
        # Convert to HSL
        hsl = self.hsl()

        # Modify light value
        if at_least is None and at_most is None:
            hsl.l = light
        else:
            if at_most:
                hsl.l = min(hsl.l, light)
            if at_least:
                hsl.l = max(hsl.l, light)

        # Convert back to RGB
        return hsl.rgb()

    def is_light(self):
        return self.hsl().l > 0.5

    def overlay(self, color: 'RGB', alpha: float) -> 'RGB':
        """
        Overlay a color on top of this color

        :param color: Overlay color
        :param alpha: Overlay alpha
        :return: New color (original isn't modified)
        """
        return RGB(*[round((1 - alpha) * v1 + alpha * v2) for v1, v2 in zip(self, color)])



# This file is automatically generated. Please do not modify.

from . import AsciiArt

fedora_small = AsciiArt(match=r'''"fedora_small"''', color='12', ascii=r"""
${c1}        ,'''''.
       |   ,.  |
       |  |  '_'
  ,....|  |..
.'  ,_;|   ..'
|  |   |  |
|  ',_,'  |
 '.     ,'
   '''''
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

ubuntu_small = AsciiArt(match=r'''"ubuntu_small"''', color='1', ascii=r"""
${c1}       ..;,; .,;,.
    .,lool: .ooooo,
   ;oo;:    .coool.
 ....         ''' ,l;
:oooo,            'oo.
looooc            :oo'
 '::'             ,oo:
   ,.,       .... co,
    lo:;.   :oooo; .
     ':ooo; cooooc
        '''  ''''
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

vanilla = AsciiArt(match=r'''"Vanilla"*''', color='11 15', ascii=r"""
${c1}

                  .OO.
                 .OOOO.
                .OO${c2}II${c1}OO.
               IOOO${c2}II${c1}OOOI
              .OOOO${c2}II${c1}OOOO.
  ............IOOOO${c2}II${c1}OOOOI............
 OOOOOOOOOOOOOOOOOO${c2}II${c1}OOOOOOOOOOOOOOOOOO
  OOOO${c2}II${c1}OOOOOOOOOOO${c2}II${c1}OOOOOOOOOOO${c2}II${c1}OOOO
   OOOOO${c2}III${c1}OOOOOOOO${c2}II${c1}OOOOOOOO${c2}III${c1}OOOOO
    OOOOOOO${c2}IIII${c1}OO${c2}I....I${c1}OO${c2}IIII${c1}OOOOOOO
     OOOOOOOOOO${c2}II..  ..II${c1}OOOOOOOOOO
       OOOOOOO${c2}II.      .II${c1}OOOOOOO
        OOOOOO${c2}II:......:II${c1}OOOOOO
       OOOOOOOOO${c2}IIIIIIII${c1}OOOOOOOOO
      OOOOOO${c2}II${c1}OOOOOOOOOOOO${c2}II${c1}OOOOOO
     OOOO${c2}II${c1}OOOOOOOI   IOOOOOO${c2}II${c1}OOOO
    IOO${c2}II${c1}OOOOOOOO:     :OOOOOOOO${c2}II${c1}OOI
   IOOOOOOOOOOII         IIOOOOOOOOOOI
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

arcolinux = AsciiArt(match=r'''"ArcoLinux"*''', color='7 4', ascii=r"""
${c2}                    /-
                   ooo:
                  yoooo/
                 yooooooo
                yooooooooo
               yooooooooooo
             .yooooooooooooo
            .oooooooooooooooo
           .oooooooarcoooooooo
          .ooooooooo-oooooooooo
         .ooooooooo-  oooooooooo
        :ooooooooo.    :ooooooooo
       :ooooooooo.      :ooooooooo
      :oooarcooo         .oooarcooo
     :ooooooooy           .ooooooooo
    :ooooooooo   ${c1}/ooooooooooooooooooo${c2}
   :ooooooooo      ${c1}.-ooooooooooooooooo.${c2}
  ooooooooo-             ${c1}-ooooooooooooo.${c2}
 ooooooooo-                 ${c1}.-oooooooooo.${c2}
ooooooooo.                     ${c1}-ooooooooo${c2}
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

shastraos = AsciiArt(match=r'''"ShastraOS"*''', color='6', ascii=r"""
${c1}                          ..,;;,'.
                       ':oo.     ;o:
                     :o,           ol
                   .oo        ..';co:
                   ooo',;:looo;
               .;lddl
             cx   .xl     .c:'
            dd     xx     xx ,d;
           .xd     cx.    xx   dd.
            cx:    .xo    xx    ,x:
             'xl    xx    cx'    .xl
               xd,  xx    .xd     dx.
                .xo:xx     xx    .xx
                   'c      xx:.'lx:
                      ..,;cxxxo
             .';:codxxl   lxo
           cd.           'xo
           :o,         'ld
            .oc'...';lo
""")
    


from __future__ import annotations

import os
import platform
import re
import shlex
import shutil
import subprocess
import sys
from dataclasses import dataclass
from pathlib import Path
from subprocess import check_output
from tempfile import TemporaryDirectory
from typing import Iterable

from .color_util import color, printc
from .constants import GLOBAL_CFG, IS_WINDOWS
from .distros import distro_detector
from .presets import ColorProfile
from .serializer import from_dict
from .types import BackendLiteral, ColorAlignMode

RE_NEOFETCH_COLOR = re.compile('\\${c[0-9]}')
SRC = Path(__file__).parent


def literal_input(prompt: str, options: Iterable[str], default: str, show_ops: bool = True) -> str:
    """
    Ask the user to provide an input among a list of options

    :param prompt: Input prompt
    :param options: Options
    :param default: Default option
    :param show_ops: Show options
    :return: Selection
    """
    options = list(options)
    lows = [o.lower() for o in options]

    if show_ops:
        op_text = '|'.join([f'&l&n{o}&L&N' if o == default else o for o in options])
        printc(f'{prompt} ({op_text})')
    else:
        printc(f'{prompt} (default: {default})')

    def find_selection(sel: str):
        if not sel:
            return None

        # Find exact match
        if sel in lows:
            return options[lows.index(sel)]

        # Find starting abbreviation
        for i, op in enumerate(lows):
            if op.startswith(sel):
                return options[i]

        return None

    selection = input('> ').lower() or default
    while not find_selection(selection):
        print(f'Invalid selection! {selection} is not one of {"|".join(options)}')
        selection = input('> ').lower() or default

    print()

    return find_selection(selection)


def term_size() -> tuple[int, int]:
    """
    Get terminal size
    :return:
    """
    try:
        return os.get_terminal_size().columns, os.get_terminal_size().lines
    except Exception:
        return 100, 20


def ascii_size(asc: str) -> tuple[int, int]:
    """
    Get distro ascii width, height ignoring color code

    :param asc: Distro ascii
    :return: Width, Height
    """
    return max(len(line) for line in re.sub(RE_NEOFETCH_COLOR, '', asc).split('\n')), len(asc.split('\n'))


def normalize_ascii(asc: str) -> str:
    """
    Make sure every line are the same width
    """
    w = ascii_size(asc)[0]
    return '\n'.join(line + ' ' * (w - ascii_size(line)[0]) for line in asc.split('\n'))


def fill_starting(asc: str) -> str:
    """
    Fill the missing starting placeholders.

    E.g. "${c1}...\n..." -> "${c1}...\n${c1}..."
    """
    new = []
    last = ''
    for line in asc.split('\n'):
        new.append(last + line)

        # Line has color placeholders
        matches = RE_NEOFETCH_COLOR.findall(line)
        if len(matches) > 0:
            # Get the last placeholder for the next line
            last = matches[-1]

    return '\n'.join(new)


@dataclass
class ColorAlignment:
    mode: ColorAlignMode

    # custom_colors[ascii color index] = unique color index in preset
    custom_colors: dict[int, int] = ()

    # Foreground/background ascii color index
    fore_back: tuple[int, int] = ()

    @classmethod
    def from_dict(cls, d: dict):
        ca = from_dict(cls, d)
        # Backward compatibility
        if type(ca.custom_colors) is not dict:
            if type(ca.custom_colors) is list:
                ca.custom_colors = {i + 1: v for i, v in enumerate(ca.custom_colors)}
            else:
                ca.custom_colors = {}
        # Fixup: Keys must json serialize as str, so we convert them back to int.
        ca.custom_colors = {int(k): v for k, v in ca.custom_colors.items()}
        return ca

    def recolor_ascii(self, asc: str, preset: ColorProfile) -> str:
        """
        Use the color alignment to recolor an ascii art

        :return Colored ascii, Uncolored lines
        """
        asc = fill_starting(asc)

        if self.fore_back and self.mode in ['horizontal', 'vertical']:
            fore, back = self.fore_back

            # Replace foreground colors
            asc = asc.replace(f'${{c{fore}}}', color('&0' if GLOBAL_CFG.is_light else '&f'))
            lines = asc.split('\n')

            # Add new colors
            if self.mode == 'horizontal':
                colors = preset.with_length(len(lines))
                asc = '\n'.join([l.replace(f'${{c{back}}}', colors[i].to_ansi()) + color('&~&*') for i, l in enumerate(lines)])
            else:
                raise NotImplementedError()

            # Remove existing colors
            asc = re.sub(RE_NEOFETCH_COLOR, '', asc)

        elif self.mode in ['horizontal', 'vertical']:
            # Remove existing colors
            asc = re.sub(RE_NEOFETCH_COLOR, '', asc)
            lines = asc.split('\n')

            # Add new colors
            if self.mode == 'horizontal':
                colors = preset.with_length(len(lines))
                asc = '\n'.join([colors[i].to_ansi() + l + color('&~&*') for i, l in enumerate(lines)])
            else:
                asc = '\n'.join(preset.color_text(line) + color('&~&*') for line in lines)

        else:
            preset = preset.unique_colors()

            # Apply colors
            color_map = {ai: preset.colors[pi].to_ansi() for ai, pi in self.custom_colors.items()}
            for ascii_i, c in color_map.items():
                asc = asc.replace(f'${{c{ascii_i}}}', c)

        return asc


def if_file(f: str | Path) -> Path | None:
    """
    Return the file if the file exists, or return none. Useful for chaining 'or's
    """
    f = Path(f)
    if f.is_file():
        return f
    return None


def get_command_path() -> str:
    """
    Get the absolute path of the neofetch command

    :return: Command path
    """
    cmd_path = (if_file(SRC.parent / 'neofetch') or if_file(SRC / 'scripts/neowofetch'))

    if not cmd_path:
        printc("&cError: Neofetch script cannot be found")
        exit(127)

    return str(cmd_path)


def ensure_git_bash() -> Path:
    """
    Ensure git bash installation for windows

    :returns git bash path
    """
    if not IS_WINDOWS:
        return Path('/usr/bin/bash')

    # Bundled git bash
    git_path = (if_file(SRC / 'git/bin/bash.exe')
                or if_file("C:/Program Files/Git/bin/bash.exe")
                or if_file("C:/Program Files (x86)/Git/bin/bash.exe"))

    if not git_path.is_file():
        printc("&cError: Git Bash installation not found")
        sys.exit(127)

    return git_path


def check_windows_cmd():
    """
    Check if this script is running under cmd.exe. If so, launch an external window with git bash
    since cmd doesn't support RGB colors.
    """
    # if IS_WINDOWS:
    #     import psutil
    #     # TODO: This line does not correctly identify cmd prompts...
    #     if psutil.Process(os.getppid()).name().lower().strip() == 'cmd.exe':
    #         print("cmd.exe doesn't support RGB colors, restarting in MinTTY...")
    #         cmd = f'"{ensure_git_bash().parent.parent / "usr/bin/mintty.exe"}" -s 110,40 -e python -m hyfetch --ask-exit'
    #         os.system(cmd)
    #         sys.exit(0)


def run_neofetch_cmd(args: str, pipe: bool = False) -> str | None:
    """
    Run neofetch command
    """
    if platform.system() != 'Windows':
        bash = ['/usr/bin/env', 'bash'] if Path('/usr/bin/env').is_file() else [shutil.which('bash')]
        full_cmd = [*bash, get_command_path(), *shlex.split(args)]

    else:
        cmd = get_command_path().replace("\\", "/").replace("C:/", "/c/")
        args = args.replace('\\', '/').replace('C:/', '/c/')

        full_cmd = [ensure_git_bash(), cmd, *shlex.split(args)]

    full_cmd = [str(c) for c in full_cmd]
    if pipe:
        return check_output(full_cmd).decode().strip()
    else:
        subprocess.run(full_cmd)


def get_distro_ascii(distro: str | None = None) -> str:
    """
    Get the distro ascii of the current distro. Or if distro is specified, get the specific distro's
    ascii art instead.

    :return: Distro ascii
    """
    if not distro and GLOBAL_CFG.override_distro:
        distro = GLOBAL_CFG.override_distro
    if GLOBAL_CFG.debug:
        print(distro)
        print(GLOBAL_CFG)

    # Try new pure-python detection method
    det = distro_detector.detect(distro or get_distro_name())
    if det is not None:
        return normalize_ascii(det.ascii)

    if GLOBAL_CFG.debug:
        printc(f"&cError: Cannot find distro {distro}")

    # Old detection method that calls neofetch
    cmd = 'print_ascii'
    if distro:
        cmd += f' --ascii_distro {distro}'

    asc = run_neofetch_cmd(cmd, True)

    # Unescape backslashes here because backslashes are escaped in neofetch for printf
    asc = asc.replace('\\\\', '\\')

    return normalize_ascii(asc)


def get_distro_name():
    return run_neofetch_cmd('ascii_distro_name', True)


def run(asc: str, backend: BackendLiteral, args: str = ''):
    if backend == "neofetch":
        return run_neofetch(asc, args)
    if backend == "fastfetch":
        return run_fastfetch(asc, args)
    if backend == "fastfetch-old":
        return run_fastfetch(asc, args, legacy=True)
    if backend == "qwqfetch":
        return run_qwqfetch(asc, args)


def run_qwqfetch(asc: str, args: str = ''):
    """
    Run qwqfetch with colors

    :param asc: Ascii art
    :param args: Additional arguments to pass to qwqfetch
    """
    asc = asc.replace('\\', '\\\\')

    # call qwqfetch to print string
    try:
        import qwqfetch
        # distro_detector only return a bash variable
        # so we use qwqfetch builtin distro detector
        print(qwqfetch.get_ascres(asc))
    except ImportError as e:  # module not found etc
        print("qwqfetch is not installed. Install it by executing:")  # use print to output hint directly
        print("pip install git+https://github.com/nexplorer-3e/qwqfetch")  # TODO: public repo
        exit(127)


def run_neofetch(asc: str, args: str = ''):
    """
    Run neofetch with colors

    :param asc: Ascii art
    :param args: Additional arguments to pass to neofetch
    """
    # Escape backslashes here because backslashes are escaped in neofetch for printf
    asc = asc.replace('\\', '\\\\')

    # Write temp file
    with TemporaryDirectory() as tmp_dir:
        tmp_dir = Path(tmp_dir)
        path = tmp_dir / 'ascii.txt'
        path.write_text(asc, 'utf-8')

        # Call neofetch with the temp file
        if args:
            args = ' ' + args
        run_neofetch_cmd(f'--ascii --source {path.absolute()} --ascii-colors' + args)


def fastfetch_path() -> Path | None:
    return (shutil.which('fastfetch')
            or if_file(SRC / 'fastfetch/usr/bin/fastfetch')
            or if_file(SRC / 'fastfetch/fastfetch')
            or if_file(SRC / 'fastfetch/fastfetch.exe'))


def run_fastfetch(asc: str, args: str = '', legacy: bool = False):
    """
    Run neofetch with colors

    :param asc: Ascii art
    :param args: Additional arguments to pass to fastfetch
    :param legacy: Set true when using fastfetch < 1.8.0
    """
    # Find fastfetch binary
    ff_path = fastfetch_path()

    if not ff_path:
        printc("&cError: fastfetch binary is not found. Please install fastfetch first.")
        exit(127)

    # Write temp file
    with TemporaryDirectory() as tmp_dir:
        tmp_dir = Path(tmp_dir)
        path = tmp_dir / 'ascii.txt'
        path.write_text(asc, 'utf-8')

        os.environ['FFTS_IGNORE_PARENT'] = '1'

        # Call fastfetch with the temp file
        proc = subprocess.run([str(ff_path), '--raw' if legacy else '--file-raw',
                               str(path.absolute()), *shlex.split(args)])
        if proc.returncode == 144:
            printc("&6Error code 144 detected: Please upgrade fastfetch to >=1.8.0 or use the 'fastfetch-old' backend")


def get_fore_back(distro: str | None = None) -> tuple[int, int] | None:
    """
    Get recommended foreground-background configuration for distro, or None if the distro ascii is
    not suitable for fore-back configuration.

    :return:
    """
    if not distro and GLOBAL_CFG.override_distro:
        distro = GLOBAL_CFG.override_distro
    if not distro:
        distro = get_distro_name().lower()
    distro = distro.lower().replace(' ', '-')
    for k, v in fore_back.items():
        if distro.startswith(k.lower()):
            return v
    return None


# Foreground-background recommendation
fore_back = {
    'fedora': (2, 1),
    'kubuntu': (2, 1),
    'lubuntu': (2, 1),
    'xubuntu': (2, 1),
    'ubuntu-cinnamon': (2, 1),
    'ubuntu-kylin': (2, 1),
    'ubuntu-mate': (2, 1),
    'ubuntu-studio': (2, 1),
    'ubuntu-sway': (2, 1),
    'ultramarine': (2, 1),
}




# This file is automatically generated. Please do not modify.

from . import AsciiArt

zorin = AsciiArt(match=r'''"Zorin"*''', color='4 6', ascii=r"""
${c1}        `osssssssssssssssssssso`
       .osssssssssssssssssssssso.
      .+oooooooooooooooooooooooo+.


  `::::::::::::::::::::::.         .:`
 `+ssssssssssssssssss+:.`     `.:+ssso`
.ossssssssssssssso/.       `-+ossssssso.
ssssssssssssso/-`      `-/osssssssssssss
.ossssssso/-`      .-/ossssssssssssssso.
 `+sss+:.      `.:+ssssssssssssssssss+`
  `:.         .::::::::::::::::::::::`


      .+oooooooooooooooooooooooo+.
       -osssssssssssssssssssssso-
        `osssssssssssssssssssso`
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

redcore = AsciiArt(match=r'''"Redcore"*''', color='1', ascii=r"""
${c1}                 RRRRRRRRR
               RRRRRRRRRRRRR
        RRRRRRRRRR      RRRRR
   RRRRRRRRRRRRRRRRRRRRRRRRRRR
 RRRRRRR  RRR         RRR RRRRRRRR
RRRRR    RR                 RRRRRRRRR
RRRR    RR     RRRRRRRR      RR RRRRRR
RRRR   R    RRRRRRRRRRRRRR   RR   RRRRR
RRRR   R  RRRRRRRRRRRRRRRRRR  R   RRRRR
RRRR     RRRRRRRRRRRRRRRRRRR  R   RRRR
 RRR     RRRRRRRRRRRRRRRRRRRR R   RRRR
  RRR    RRRRRRRRRRRRRRRRRRRR    RRRR
    RR   RRRRRRRRRRRRRRRRRRR    RRR
     RR   RRRRRRRRRRRRRRRRR    RRR
       RR   RRRRRRRRRRRRRR   RR
         R       RRRR      RR
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

ubuntu_budgie = AsciiArt(match=r'''"Ubuntu Budgie"* | "Ubuntu-Budgie"*''', color='4 7 1', ascii=r"""
${c2}           ./oydmMMMMMMmdyo/.
        :smMMMMMMMMMMMhs+:++yhs:
     `omMMMMMMMMMMMN+`        `odo`
    /NMMMMMMMMMMMMN-            `sN/
  `hMMMMmhhmMMMMMMh               sMh`
 .mMmo-     /yMMMMm`              `MMm.
 mN/       yMMMMMMMd-              MMMm
oN-        oMMMMMMMMMms+//+o+:    :MMMMo
m/          +NMMMMMMMMMMMMMMMMm. :NMMMMm
M`           .NMMMMMMMMMMMMMMMNodMMMMMMM
M-            sMMMMMMMMMMMMMMMMMMMMMMMMM
mm`           mMMMMMMMMMNdhhdNMMMMMMMMMm
oMm/        .dMMMMMMMMh:      :dMMMMMMMo
 mMMNyo/:/sdMMMMMMMMM+          sMMMMMm
 .mMMMMMMMMMMMMMMMMMs           `NMMMm.
  `hMMMMMMMMMMM.oo+.            `MMMh`
    /NMMMMMMMMMo                sMN/
     `omMMMMMMMMy.            :dmo`
        :smMMMMMMMh+-`   `.:ohs:
           ./oydmMMMMMMdhyo/.
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

salix = AsciiArt(match=r'''"Salix"*''', color='2', ascii=r'''
${c1}              __s_aaaaaaaaauuoXSSSSSSSS:
          ._xSSSSSSSSSSSSSSSSSSSSSSSSSS:
        _aSSSSSSSSSSSSSSSSSSSSSSSSSSSSS:
      _xSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS:
     <XSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS:
    -"^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'

  .ssssssssssssssssssssssssssssssssssss
  {SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSl
  oSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS;
 :XSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS;
 {SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS
 -"""""""""""""""""""""""""""""""""""'

 <assssssssssssssssssssssssssssssss>
 nSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS}
 nSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS}`
 XSSSSSSSSSSSSSSSSSSSSSSSSSSSS"`
 SSSSSSSSSSSSSSSSSSSSSSSSS!"`
-""""""""""""""""""""""`
''')
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

chaletos = AsciiArt(match=r'''"ChaletOS"*''', color='4 7 1', ascii=r"""
${c1}             `.//+osso+/:``
         `/sdNNmhyssssydmNNdo:`
       :hNmy+-`          .-+hNNs-
     /mMh/`       `+:`       `+dMd:
   .hMd-        -sNNMNo.  /yyy  /mMs`
  -NM+       `/dMd/--omNh::dMM   `yMd`
 .NN+      .sNNs:/dMNy:/hNmo/s     yMd`
 hMs    `/hNd+-smMMMMMMd+:omNy-    `dMo
:NM.  .omMy:/hNMMMMMMMMMMNy:/hMd+`  :Md`
/Md` `sm+.omMMMMMMMMMMMMMMMMd/-sm+  .MN:
/Md`      MMMMMMMMMMMMMMMMMMMN      .MN:
:NN.      MMMMMMm....--NMMMMMN      -Mm.
`dMo      MMMMMMd      mMMMMMN      hMs
 -MN:     MMMMMMd      mMMMMMN     oMm`
  :NM:    MMMMMMd      mMMMMMN    +Mm-
   -mMy.  mmmmmmh      dmmmmmh  -hMh.
     oNNs-                    :yMm/
      .+mMdo:`            `:smMd/`
         -ohNNmhsoo++osshmNNh+.
            `./+syyhhyys+:``
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

univalent = AsciiArt(match=r'''"Univalent"*''', color='6 6', ascii=r"""
${c1}
   UUUUUUU                   UUUUUUU
   UUUUUUU                   UUUUUUU
   UUUUUUU         A         UUUUUUU
   UUUUUUU        A|A        UUUUUUU
   UUUUUUU       A | A       UUUUUUU
   UUUUUUU      A  |  A      UUUUUUU
   UUUUUUU     A|  |  |A     UUUUUUU
   UUUUUUU    A |  |  | A    UUUUUUU
   UUUUUUU    A |  |  | A    UUUUUUU
   UUUUUUU    A |  |  | A    UUUUUUU
   UUUUUUU    A |  |  | A    UUUUUUU
   UUUUUUU    A |  |  | A    UUUUUUU
   UUUUUUU    A |  |  | A    UUUUUUU
    UUUUUUU   A |  |  | A   UUUUUUU
     UUUUUUU  A |  |  | A  UUUUUUU
       UUUUUUUAAAAAAAAAAAUUUUUUU
          UUUUUUUUUUUUUUUUUUU
             UUUUUUUUUUUUU
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

mer = AsciiArt(match=r'''"Mer"*''', color='4 7 1', ascii=r"""
${c1}                         dMs
                         .-`
                       `y`-o+`
                        ``NMMy
                      .--`:++.
                    .hNNNNs
                    /MMMMMN
                    `ommmd/ +/
                      ````  +/
                     `:+sssso/-`
  .-::. `-::-`     `smNMNmdmNMNd/      .://-`
.ymNMNNdmNMMNm+`  -dMMh:.....+dMMs   `sNNMMNo
dMN+::NMMy::hMM+  mMMo `ohhy/ `dMM+  yMMy::-
MMm   yMM-  :MMs  NMN` `:::::--sMMh  dMM`
MMm   yMM-  -MMs  mMM+ `ymmdsymMMMs  dMM`
NNd   sNN-  -NNs  -mMNs-.--..:dMMh`  dNN
---   .--`  `--.   .smMMmdddmMNdo`   .--
                     ./ohddds+:`
                     +h- `.:-.
                     ./`.dMMMN+
                        +MMMMMd
                        `+dmmy-
                      ``` .+`
                     .dMNo-y.
                     `hmm/
                         .:`
                         dMs
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

galliumos = AsciiArt(match=r'''"GalliumOS"*''', color='4 7 1', ascii=r"""
${c1}sooooooooooooooooooooooooooooooooooooo+:
yyooooooooooooooooooooooooooooooooo+/:::
yyysoooooooooooooooooooooooooooo+/::::::
yyyyyoooooooooooooooooooooooo+/:::::::::
yyyyyysoooooooooooooooooo++/::::::::::::
yyyyyyysoooooooooooooo++/:::::::::::::::
yyyyyyyyysoooooo${c2}sydddys${c1}+/:::::::::::::::
yyyyyyyyyysooo${c2}smMMMMMMMNd${c1}+::::::::::::::
yyyyyyyyyyyyo${c2}sMMMMMMMMMMMN${c1}/:::::::::::::
yyyyyyyyyyyyy${c2}dMMMMMMMMMMMM${c1}o//:::::::::::
yyyyyyyyyyyyy${c2}hMMMMMMMMMMMm${c1}--//::::::::::
yyyyyyyyyyyyyy${c2}hmMMMMMMMNy${c1}:..-://::::::::
yyyyyyyyyyyyyyy${c2}yyhhyys+:${c1}......://:::::::
yyyyyyyyyyyyyyys+:--...........-///:::::
yyyyyyyyyyyys+:--................://::::
yyyyyyyyyo+:-.....................-//:::
yyyyyyo+:-..........................://:
yyyo+:-..............................-//
o/:-...................................:
""")
    


# This file is automatically generated. Please do not modify.

from . import AsciiArt

gloire = AsciiArt(match=r'''"Gloire"*''', color='5 7 0', ascii=r"""
${c3}
                   &#BGPPPPPG#&
                B5?77!!?YJJ7!7YBB&
             &G5YJ77!7JYYYYYBPJ&PY#
           #PYYYYYY?!?YYYYY7?7JP5JJ
          B?YYYYYY7!!7JYYYYJ!!?JJJ5
   &&    B7?J?77?7!!!!!77777!7Y5YYBBPGGG&
  G77?YBB!!!!!!!!!!!!!JYJ??7JYJJY# PYPPG&
  J777JB?!7JJ???!!!7?JYYYYYPJ!7JB
  GYYG #JJJJJ??7!!!JYYY5PGB&GB&
     #Y!?GB5YYJY5PG###&
     GJJP
""")
    